﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls


Partial Public Class AnnotationDialog
    Inherits UserControl
    Implements ITombstonable
    Private _annotation As Annotation

    Public Shared ReadOnly CaptionProperty As DependencyProperty = DependencyProperty.Register("Caption", GetType(String), GetType(AnnotationDialog), New PropertyMetadata(AddressOf OnCaptionPropertyChanged))

    Public Event DialogDismissed As EventHandler(Of CancelEventArgs)

    Public Sub New()
        InitializeComponent()

        AddHandler Loaded, Sub(sender, args) noteTextBox.Focus()
    End Sub

    Public Property Annotation() As Annotation
        Set(ByVal value As Annotation)
            _annotation = value
            textTextBox.Text = _annotation.Text
            noteTextBox.Text = _annotation.Note
            noteTextBox.Select(noteTextBox.Text.Length, 0)
        End Set
        Get
            Return _annotation
        End Get
    End Property

    Public Property Caption() As String
        Set(ByVal value As String)
            SetValue(CaptionProperty, value)
        End Set
        Get
            Return CStr(GetValue(CaptionProperty))
        End Get
    End Property

    Public Sub SaveState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.SaveState
        state(prefix & "annotation") = Me.Annotation
        state(prefix & "text") = textTextBox.Text
        state(prefix & "note") = noteTextBox.Text
        state(prefix & "selectionStart") = noteTextBox.SelectionStart
        state(prefix & "selectionLength") = noteTextBox.SelectionLength
    End Sub

    Public Sub RestoreState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.RestoreState
        Me.Annotation = TryCast(state(prefix & "annotation"), Annotation)
        textTextBox.Text = TryCast(state(prefix & "text"), String)
        noteTextBox.Text = TryCast(state(prefix & "note"), String)
        noteTextBox.SelectionStart = CInt(Fix(state(prefix & "selectionStart")))
        noteTextBox.SelectionLength = CInt(Fix(state(prefix & "selectionLength")))
    End Sub

    Private Shared Sub OnCaptionPropertyChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, AnnotationDialog).captionTextBlock.Text = TryCast(args.NewValue, String)
    End Sub

    Private Sub OnCancelButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        RaiseEvent DialogDismissed(Me, New CancelEventArgs(True))
    End Sub

    Private Sub OnOkButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Me.Annotation.Note = noteTextBox.Text

        RaiseEvent DialogDismissed(Me, New CancelEventArgs())
    End Sub
End Class
