﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Xml.Serialization


Public Class AppSettings
    Public Shared DefaultFontFamily As String = "Segoe WP"
    Public Shared MinimumFontSizeMultiplier As Double = 0.5
    Public Shared MaximumFontSizeMultiplier As Double = 2
    Public Shared DefaultFontSizeMultiplier As Double = 1
    Public Shared BedsideMaxCount As Integer = 6

    Public Sub New()
        Me.Bedside = New ObservableCollection(Of BookInfo)()
        Me.Library = New Library()
        Me.FontFamily = DefaultFontFamily
        Me.FontSizeMultiplier = DefaultFontSizeMultiplier
        Me.PageTransition = "slide"
    End Sub

    Public Property Bedside() As ObservableCollection(Of BookInfo)

    Public Property Library() As Library

    Public Property FontFamily() As String

    Public Property FontSizeMultiplier() As Double

    Public Property PageTransition() As String

    Public Shared Function Load(ByVal filename As String) As AppSettings
        Dim appSettings As AppSettings = Nothing

        Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
            If storage.FileExists(filename) Then
                Using stream As IsolatedStorageFileStream = storage.OpenFile(filename, FileMode.Open, FileAccess.Read)
                    Dim serializer As New XmlSerializer(GetType(AppSettings))
                    appSettings = TryCast(serializer.Deserialize(stream), AppSettings)
                End Using

            Else
                appSettings = New AppSettings()
            End If
        End Using
        Return appSettings
    End Function

    Public Sub Save(ByVal filename As String)
        Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
            Using stream As IsolatedStorageFileStream = storage.OpenFile(filename, FileMode.Create, FileAccess.Write)
                Dim serializer As New XmlSerializer(GetType(AppSettings))
                serializer.Serialize(stream, Me)
            End Using
        End Using
    End Sub
End Class

