﻿Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports Microsoft.Phone.Controls


Partial Public Class BookViewer
    ' Gesture control
    Private gestureAllowed, canGoNextPage, canGoPrevPage, gestureHadFlick As Boolean
    Private viewerDisplayMode As ViewerDisplayMode = ViewerDisplayMode.Normal
    Private viewerTouchMode As ViewerTouchMode = ViewerTouchMode.Reading

    ' Field for dragging pages
    Private dragFactor As Double

    ' Field for pinch-and-pan
    Private ptPinchStart As Point

    ' Fields for text selection
    Private selectedParagraphIndex As Integer
    Private selectedTextBlock1, selectedTextBlock2 As TextBlock

    Private Sub OnGestureListenerGestureBegin(ByVal sender As Object, ByVal args As GestureEventArgs)
        ' Initialize all four Boolean fields
        gestureAllowed = viewerTouchMode = ViewerTouchMode.Reading ' ie, != ViewerTouchMode.Animating

        If Not gestureAllowed Then
            Return
        End If

        gestureHadFlick = False
        canGoNextPage = pageHosts((pageHostBaseIndex + 1) Mod 3).Child IsNot Nothing
        canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerTap(ByVal sender As Object, ByVal args As GestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        ' If we're in a zoomed state, undo the zoom
        If viewerDisplayMode = ViewerDisplayMode.Zoomed Then
            AnimatePinchAndPanUndo()
            ' Otherwise go to the next page if possible
        ElseIf canGoNextPage Then
            AnimateToNextPage()
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerHold(ByVal sender As Object, ByVal args As GestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        ' For Hold, switch to Selecting mode
        If viewerTouchMode = ViewerTouchMode.Reading AndAlso TypeOf args.OriginalSource Is TextBlock Then ' Won't be anything else, actually
            ' Get the TextBlock underneath the finger
            Dim txtblk As TextBlock = TryCast(args.OriginalSource, TextBlock)
            Dim word As WordInfo = TryCast(txtblk.Tag, WordInfo)

            ' This means it's part of the text
            If word IsNot Nothing Then
                ' Initiate selection mode
                selectedParagraphIndex = word.ParagraphIndex
                selectedTextBlock1 = txtblk
                selectedTextBlock2 = txtblk
                SelectTextBlockRange(selectedTextBlock1, selectedTextBlock2, True)

                viewerTouchMode = ViewerTouchMode.Selecting
            End If
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerFlick(ByVal sender As Object, ByVal args As FlickGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        ' Zip over to the edge
        If viewerDisplayMode = ViewerDisplayMode.Zoomed Then
            ConsolidateTransforms()
            Dim translateX As Double = 0
            Dim translateY As Double = 0

            If args.Angle < 45 Then ' flick to the right
                translateX += 100000
            ElseIf args.Angle < 135 Then ' flick to the bottom
                translateY += 100000
            ElseIf args.Angle < 225 Then ' flick to the left
                translateX -= 100000
            ElseIf args.Angle < 315 Then ' flick to the top
                translateY -= 100000
            Else ' flick to the right
                translateX += 100000
            End If

            ClampPinchAndPanTransforms(1, translateX, translateY)
            ConsolidateTransforms()
            ' This is the logic for modes of Reading (a Flick w/o a Drag)
            '      or Dragging (a Flick after a Drag)
        ElseIf viewerTouchMode <> ViewerTouchMode.Selecting Then
            If args.Angle < 45 OrElse args.Angle > 315 Then
                If Not canGoPrevPage Then
                    If Me.ChapterIndex <> 0 AndAlso Me.PageIndex = 0 Then
                        pageHosts((pageHostBaseIndex + 2) Mod 3).Child = PageProvider.GetLastPage(prevPageChapterIndex, prevPagePageIndex)
                        canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
                    End If
                End If
                If canGoPrevPage Then
                    gestureHadFlick = True
                    AnimateToPreviousPage()
                End If

            ElseIf args.Angle > 135 AndAlso args.Angle < 225 AndAlso canGoNextPage Then
                gestureHadFlick = True
                AnimateToNextPage()
                ' Generate BookmarkInserted event
            Else
                RaiseEvent BookmarkInserted(Me, EventArgs.Empty)
            End If
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerDragStarted(ByVal sender As Object, ByVal args As DragStartedGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        If viewerTouchMode = ViewerTouchMode.Selecting Then
        ElseIf viewerDisplayMode = ViewerDisplayMode.Zoomed Then
            viewerTouchMode = ViewerTouchMode.Panning
        ElseIf viewerTouchMode = ViewerTouchMode.Reading Then
            viewerTouchMode = ViewerTouchMode.Dragging
            dragFactor = 0
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerDragDelta(ByVal sender As Object, ByVal args As DragDeltaGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        Select Case viewerTouchMode
            Case ViewerTouchMode.Dragging
                Dim newDragFactor As Double = dragFactor - args.HorizontalChange / pageHosts(pageHostBaseIndex).ActualWidth

                If newDragFactor < 0 AndAlso (Not canGoPrevPage) AndAlso Me.ChapterIndex <> 0 AndAlso Me.PageIndex = 0 Then
                    pageHosts((pageHostBaseIndex + 2) Mod 3).Child = PageProvider.GetLastPage(prevPageChapterIndex, prevPagePageIndex)
                    canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
                End If

                Dim maximum As Double = If(canGoNextPage, 1, 0)
                Dim minimum As Double = If(canGoPrevPage, -1, 0)
                dragFactor = Math.Max(minimum, Math.Min(maximum, newDragFactor))
                Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex + dragFactor

            Case ViewerTouchMode.Selecting
                ' Find TextBlock under touch point
                Dim pt As Point = args.GetPosition(Application.Current.RootVisual)
                Dim hitTestElements As IEnumerable(Of UIElement) = VisualTreeHelper.FindElementsInHostCoordinates(pt, pageHosts(pageHostBaseIndex))

                For Each hitTestElement In hitTestElements
                    If TypeOf hitTestElement Is TextBlock Then
                        Dim txtblk As TextBlock = TryCast(hitTestElement, TextBlock)
                        Dim word As WordInfo = TryCast(txtblk.Tag, WordInfo)

                        ' Extend the selection to that word
                        If word IsNot Nothing AndAlso word.ParagraphIndex = selectedParagraphIndex Then
                            selectedTextBlock2 = txtblk
                            SelectTextBlockRange(selectedTextBlock1, selectedTextBlock2, True)
                            Exit For
                        End If
                    End If
                Next hitTestElement

            Case ViewerTouchMode.Panning
                ' Set translation transform only
                Dim translateX As Double = translateTransform.X + args.HorizontalChange
                Dim translateY As Double = translateTransform.Y + args.VerticalChange
                ClampPinchAndPanTransforms(1, translateX, translateY)
        End Select
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerDragCompleted(ByVal sender As Object, ByVal args As DragCompletedGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        Select Case viewerTouchMode
            ' This logic is in GestureCompleted
            Case ViewerTouchMode.Dragging, ViewerTouchMode.Selecting

            Case ViewerTouchMode.Panning
                ' Transfer Scale and Translate transforms to Matrix transform
                ConsolidateTransforms()
        End Select
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerPinchStarted(ByVal sender As Object, ByVal args As PinchStartedGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        If viewerTouchMode = ViewerTouchMode.Reading Then
            ' Find scaling center, subject to current transform
            Dim ptScaleCenter As Point = args.GetPosition(manipulationGrid)
            ptScaleCenter = matrixTransform.Transform(ptScaleCenter)
            scaleTransform.CenterX = ptScaleCenter.X
            scaleTransform.CenterY = ptScaleCenter.Y

            ' Save initial point
            ptPinchStart = args.GetPosition(Me)
            viewerDisplayMode = ViewerDisplayMode.Zoomed
            viewerTouchMode = ViewerTouchMode.Pinching
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerPinchDelta(ByVal sender As Object, ByVal args As PinchGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        If viewerTouchMode = ViewerTouchMode.Pinching Then
            ' Get raw transform values
            Dim ptPosition As Point = args.GetPosition(Me)
            Dim translateX As Double = ptPosition.X - ptPinchStart.X
            Dim translateY As Double = ptPosition.Y - ptPinchStart.Y
            Dim scale As Double = args.DistanceRatio

            ' Don't allow the page edges to be visible
            ClampPinchAndPanTransforms(scale, translateX, translateY)
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerPinchCompleted(ByVal sender As Object, ByVal args As PinchGestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        If viewerTouchMode = ViewerTouchMode.Pinching Then
            ' Transfer Scale and Translate transforms to Matrix transform
            ConsolidateTransforms()
        End If
        args.Handled = True
    End Sub

    Private Sub OnGestureListenerGestureCompleted(ByVal sender As Object, ByVal args As GestureEventArgs)
        If Not gestureAllowed Then
            Return
        End If

        Select Case viewerTouchMode
            Case ViewerTouchMode.Dragging
                If Not gestureHadFlick Then
                    If dragFactor > 0.5 Then
                        AnimateToNextPage()

                    ElseIf dragFactor < -0.5 Then
                        AnimateToPreviousPage()

                    Else
                        AnimateToSamePage()
                    End If
                End If

            Case ViewerTouchMode.Selecting
                ' Fire the TextSelected event
                If TextSelectedEvent IsNot Nothing Then
                    Dim panel As Panel = TryCast(selectedTextBlock1.Parent, Panel)
                    Dim index1 As Integer = panel.Children.IndexOf(selectedTextBlock1)
                    Dim index2 As Integer = panel.Children.IndexOf(selectedTextBlock2)
                    Dim minIndex As Integer = Math.Min(index1, index2)
                    Dim maxIndex As Integer = Math.Max(index1, index2)
                    Dim txtblk1 As TextBlock = (TryCast(panel.Children(minIndex), TextBlock))
                    Dim txtblk2 As TextBlock = (TryCast(panel.Children(maxIndex), TextBlock))
                    Dim word1 As WordInfo = TryCast(txtblk1.Tag, WordInfo)
                    Dim word2 As WordInfo = TryCast(txtblk2.Tag, WordInfo)

                    ' Get the selected text && its minimum and maximum positions
                    Dim text As String = ""
                    Dim ulAll As New Point(Double.MaxValue, Double.MaxValue)
                    Dim lrAll As New Point(Double.MinValue, Double.MinValue)

                    For index = minIndex To maxIndex
                        Dim txtblk As TextBlock = TryCast(panel.Children(index), TextBlock)

                        If txtblk IsNot Nothing Then
                            text &= txtblk.Text

                            If index < maxIndex Then
                                text &= " "
                            End If

                            Dim ulText As Point = txtblk.TransformToVisual(Me).Transform(New Point())
                            Dim lrText As Point = txtblk.TransformToVisual(Me).Transform(New Point(txtblk.ActualWidth, txtblk.ActualHeight))

                            ulAll.X = Math.Min(ulAll.X, ulText.X)
                            ulAll.Y = Math.Min(ulAll.Y, ulText.Y)
                            lrAll.X = Math.Max(lrAll.X, lrText.X)
                            lrAll.Y = Math.Max(lrAll.Y, lrText.Y)
                        End If
                    Next index

                    ' Set up the event arguments
                    Dim eventArgs As TextSelectedEventArgs = New TextSelectedEventArgs With {.ChapterIndex = Me.ChapterIndex, .ParagraphIndex = selectedParagraphIndex, .CharacterIndex = word1.CharacterIndex, .CharacterCount = word2.CharacterIndex - word1.CharacterIndex, .Text = text, .Bounds = New Rect(ulAll, lrAll)}

                    ' Fire the event!
                    RaiseEvent TextSelected(Me, eventArgs)
                    viewerTouchMode = viewerTouchMode.Reading
                End If

            Case ViewerTouchMode.Pinching, ViewerTouchMode.Panning
                viewerTouchMode = ViewerTouchMode.Reading ' but DisplayMode is still Zoomed!
        End Select

        ' Abandon the zoom if the user has restored it to close to actual size
        If viewerDisplayMode = ViewerDisplayMode.Zoomed Then
            If matrixTransform.Matrix.M11 < 1.05 Then
                AnimatePinchAndPanUndo()
            End If
        End If

        args.Handled = True
    End Sub
End Class

