﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports Microsoft.Phone.Controls


Partial Public Class BookViewer
    Inherits UserControl
    Private Shared ReadOnly pinchUndoAnimationDuration As TimeSpan = TimeSpan.FromSeconds(0.5)

    Private pageHosts(2) As Border
    Private pageHostBaseIndex As Integer = 0 ' current page

    Private nextPageChapterIndex As Integer
    Private nextPagePageIndex As Integer
    Private prevPageChapterIndex As Integer
    Private prevPagePageIndex As Integer

    Public Shared ReadOnly PageProviderProperty As DependencyProperty = DependencyProperty.Register("PageProvider", GetType(IPageProvider), GetType(BookViewer), New PropertyMetadata(AddressOf OnPageProviderChanged))

    Public Shared ReadOnly PageTransitionProperty As DependencyProperty = DependencyProperty.Register("PageTransition", GetType(PageTransition), GetType(BookViewer), New PropertyMetadata(AddressOf OnPageTransitionChanged))

    Public Shared ReadOnly ChapterIndexProperty As DependencyProperty = DependencyProperty.Register("ChapterIndex", GetType(Integer), GetType(BookViewer), New PropertyMetadata(0, AddressOf OnPageChanged))

    Public Shared ReadOnly PageIndexProperty As DependencyProperty = DependencyProperty.Register("PageIndex", GetType(Integer), GetType(BookViewer), New PropertyMetadata(0, AddressOf OnPageChanged))

    ' Public events
    Public Event PageChanged As EventHandler
    Public Event TextSelected As EventHandler(Of TextSelectedEventArgs)
    Public Event BookmarkInserted As EventHandler

    Public Sub New()
        InitializeComponent()

        pageHosts(0) = pageHost0
        pageHosts(1) = pageHost1
        pageHosts(2) = pageHost2
    End Sub

    Public Property PageProvider() As IPageProvider
        Set(ByVal value As IPageProvider)
            SetValue(PageProviderProperty, value)
        End Set
        Get
            Return CType(GetValue(PageProviderProperty), IPageProvider)
        End Get
    End Property

    Public Property PageTransition() As PageTransition
        Set(ByVal value As PageTransition)
            SetValue(PageTransitionProperty, value)
        End Set
        Get
            Return CType(GetValue(PageTransitionProperty), PageTransition)
        End Get
    End Property

    Public Property ChapterIndex() As Integer
        Protected Set(ByVal value As Integer)
            SetValue(ChapterIndexProperty, value)
        End Set
        Get
            Return CInt(Fix(GetValue(ChapterIndexProperty)))
        End Get
    End Property

    Public Property PageIndex() As Integer
        Protected Set(ByVal value As Integer)
            SetValue(PageIndexProperty, value)
        End Set
        Get
            Return CInt(Fix(GetValue(PageIndexProperty)))
        End Get
    End Property

    Public Function JumpToPage(ByVal chapterIndex As Integer, ByVal pageIndex As Integer) As Boolean
        ' Save chapter and page indices in public properties
        Me.ChapterIndex = chapterIndex
        Me.PageIndex = pageIndex

        If Me.PageProvider IsNot Nothing Then
            Dim invalidated As Boolean = Me.PageProvider.SetPageSize(New Size(pageHost0.ActualWidth, pageHost0.ActualHeight))
            invalidated = invalidated Or Me.PageProvider.SetFont(Me.FontFamily, Me.FontSize)

            ' need to return if these have invalidated pagination information
            If invalidated Then
                Return False
            End If

            Dim element As FrameworkElement = Me.PageProvider.GetPage(chapterIndex, pageIndex)

            If element IsNot Nothing Then
                ' Set pageHosts for this page and the next page
                pageHosts(pageHostBaseIndex).Child = element
                SetUpNextPage()
                SetUpPreviousPage()

                ' Set the FractionalBaseIndex for the PageTransition
                Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex
                Return True
            End If
        End If
        Return False
    End Function

    Public Function JumpToCharacterOffset(ByVal characterOffset As Integer) As Boolean
        ' Ensure that these properties actually change
        Me.ChapterIndex = 0
        Me.PageIndex = 0

        If Me.PageProvider IsNot Nothing Then
            Me.PageProvider.SetPageSize(New Size(pageHost0.ActualWidth, pageHost0.ActualHeight))
            Me.PageProvider.SetFont(Me.FontFamily, Me.FontSize)
            Dim chapterIndex As Integer
            Dim pageIndex As Integer
            Dim element As FrameworkElement = Me.PageProvider.GetPage(characterOffset, chapterIndex, pageIndex)

            If element IsNot Nothing Then
                Me.ChapterIndex = chapterIndex
                Me.PageIndex = pageIndex

                ' Set pageHosts for this page and the next page
                pageHosts(pageHostBaseIndex).Child = element
                SetUpNextPage()
                SetUpPreviousPage()

                ' Set the FractionlBaseIndex for the PageTransition
                Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex
                Return True
            End If
        End If
        Return False
    End Function

    Public Sub HighlightText(ByVal findInfo As FindInfo)
        Dim pageHost As FrameworkElement = pageHosts(pageHostBaseIndex)
        HighlightText(pageHost, findInfo)
    End Sub

    Private Sub HighlightText(ByVal parent As FrameworkElement, ByVal findInfo As FindInfo)
        If TypeOf parent Is TextBlock AndAlso TypeOf (TryCast(parent, TextBlock)).Tag Is WordInfo Then
            Dim txtblk As TextBlock = TryCast(parent, TextBlock)
            Dim wordInfo As WordInfo = TryCast(txtblk.Tag, WordInfo)

            If wordInfo.ParagraphIndex = findInfo.ParagraphIndex AndAlso wordInfo.CharacterIndex + wordInfo.CharacterCount > findInfo.StartCharacterIndex AndAlso wordInfo.CharacterIndex <= findInfo.EndCharacterIndex Then
                txtblk.Foreground = TryCast(Me.Resources("PhoneAccentBrush"), Brush)
                findInfo.WordLocation = wordInfo.LocationTop
            End If
        End If

        Dim numChildren As Integer = VisualTreeHelper.GetChildrenCount(parent)

        For i = 0 To numChildren - 1
            Dim child As FrameworkElement = TryCast(VisualTreeHelper.GetChild(parent, i), FrameworkElement)

            If child IsNot Nothing Then
                HighlightText(child, findInfo)
            End If
        Next i
    End Sub

    Public Sub ClearHighlighting()
        Dim pageHost As FrameworkElement = pageHosts(pageHostBaseIndex)
        ClearHighlighting(pageHost)
    End Sub

    Private Sub ClearHighlighting(ByVal parent As FrameworkElement)
        If TypeOf parent Is TextBlock AndAlso TypeOf (TryCast(parent, TextBlock)).Tag Is WordInfo Then
            TryCast(parent, TextBlock).ClearValue(TextBlock.ForegroundProperty)
        End If

        Dim numChildren As Integer = VisualTreeHelper.GetChildrenCount(parent)

        For i = 0 To numChildren - 1
            Dim child As FrameworkElement = TryCast(VisualTreeHelper.GetChild(parent, i), FrameworkElement)

            If child IsNot Nothing Then
                ClearHighlighting(child)
            End If
        Next i
    End Sub

    Private Sub SetUpNextPage()
        nextPageChapterIndex = Me.ChapterIndex
        nextPagePageIndex = Me.PageIndex + 1
        Dim element As FrameworkElement = Me.PageProvider.GetPage(nextPageChapterIndex, nextPagePageIndex)

        If element Is Nothing Then
            nextPageChapterIndex += 1
            nextPagePageIndex = 0
            element = Me.PageProvider.GetPage(nextPageChapterIndex, nextPagePageIndex)
        End If

        pageHosts((pageHostBaseIndex + 1) Mod 3).Child = element
    End Sub

    Private Sub SetUpPreviousPage()
        prevPageChapterIndex = Me.ChapterIndex
        prevPagePageIndex = Me.PageIndex - 1
        Dim element As FrameworkElement = Nothing

        If prevPagePageIndex <> -1 Then
            element = Me.PageProvider.GetPage(prevPageChapterIndex, prevPagePageIndex)
        ElseIf prevPageChapterIndex <> 0 Then
            prevPageChapterIndex -= 1
            element = Me.PageProvider.QueryLastPage(prevPageChapterIndex, prevPagePageIndex)
        End If
        pageHosts((pageHostBaseIndex + 2) Mod 3).Child = element
    End Sub

    Private Sub ClampPinchAndPanTransforms(ByVal scale As Double, ByVal translateX As Double, ByVal translateY As Double)
        ' This is the matrix transform from previous operations
        calcMatrixTransform.Matrix = matrixTransform.Matrix

        ' Calculate scaling factor so it's always 1 or greater
        Dim totalScale As Double = scale * matrixTransform.Matrix.M11
        totalScale = Math.Max(1, totalScale)
        scale = totalScale / matrixTransform.Matrix.M11

        ' Set up properties for new scale matrix
        calcScaleTransform.CenterX = scaleTransform.CenterX
        calcScaleTransform.CenterY = scaleTransform.CenterY
        calcScaleTransform.ScaleX = scale
        calcScaleTransform.ScaleY = scale

        ' Set up properties for new translation matrix
        calcTranslateTransform.X = translateX
        calcTranslateTransform.Y = translateY

        ' Obtain the total matrix from the transform group
        Dim totalMatrix As Matrix = calcTransformGroup.Value

        ' Restict translation to original area of transformed element
        Dim totalTranslationX As Double = totalMatrix.OffsetX
        Dim totalTranslationY As Double = totalMatrix.OffsetY

        Dim clampedTranslationX As Double = Math.Min(0, Math.Max((1 - totalMatrix.M11) * manipulationGrid.ActualWidth, totalTranslationX))

        Dim clampedTranslationY As Double = Math.Min(0, Math.Max((1 - totalMatrix.M22) * manipulationGrid.ActualHeight, totalTranslationY))

        ' Adjust translation factors
        translateX += clampedTranslationX - totalTranslationX
        translateY += clampedTranslationY - totalTranslationY

        ' Set transforms
        scaleTransform.ScaleX = scale
        scaleTransform.ScaleY = scale
        translateTransform.X = translateX
        translateTransform.Y = translateY
    End Sub

    Private Sub ConsolidateTransforms()
        matrixTransform.Matrix = transformGroup.Value
        scaleTransform.ScaleY = 1
        scaleTransform.ScaleX = scaleTransform.ScaleY
        scaleTransform.CenterY = 0
        scaleTransform.CenterX = scaleTransform.CenterY
        translateTransform.Y = 0
        translateTransform.X = translateTransform.Y
    End Sub

    ' Highlight all words in range if select == true; unhighlight all other words on page
    Private Sub SelectTextBlockRange(ByVal txtblk1 As TextBlock, ByVal txtblk2 As TextBlock, ByVal [select] As Boolean)
        Dim panel As Panel = TryCast(txtblk1.Parent, Panel)
        Dim index1 As Integer = panel.Children.IndexOf(txtblk1)
        Dim index2 As Integer = panel.Children.IndexOf(txtblk2)
        Dim minIndex As Integer = Math.Min(index1, index2)
        Dim maxIndex As Integer = Math.Max(index1, index2)
        Dim highlightBrush As Brush = TryCast(Me.Resources("PhoneAccentBrush"), Brush)

        For index = 0 To panel.Children.Count - 1
            Dim txtblk As TextBlock = TryCast(panel.Children(index), TextBlock)

            If txtblk IsNot Nothing Then
                If [select] AndAlso index >= minIndex AndAlso index <= maxIndex Then
                    txtblk.Foreground = highlightBrush
                Else
                    txtblk.ClearValue(TextBlock.ForegroundProperty)
                End If
            End If
        Next index
    End Sub

    Public Sub AnimateToNextPage()
        Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex + 1, .Duration = CalculateFractionalDuration(pageHostBaseIndex + 1)}

        Storyboard.SetTarget(anima, Me.PageTransition)
        Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

        Dim storyboardVariable As New Storyboard()
        storyboardVariable.Children.Add(anima)

        AddHandler storyboardVariable.Completed, Sub(sender As Object, args As EventArgs)
                                                     pageHostBaseIndex = (pageHostBaseIndex + 1) Mod 3
                                                     Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex

                                                     ' Shift the indices from current to previous
                                                     prevPageChapterIndex = Me.ChapterIndex
                                                     prevPagePageIndex = Me.PageIndex

                                                     ' Shift the indices from next to current
                                                     Me.ChapterIndex = nextPageChapterIndex
                                                     Me.PageIndex = nextPagePageIndex

                                                     ' Set up the new next page
                                                     SetUpNextPage()
                                                     viewerTouchMode = viewerTouchMode.Reading
                                                 End Sub

        viewerTouchMode = ViewerTouchMode.Animating
        storyboardVariable.Begin()
    End Sub

    Private Sub AnimateToPreviousPage()
        Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex - 1, .Duration = CalculateFractionalDuration(pageHostBaseIndex - 1)}

        Storyboard.SetTarget(anima, Me.PageTransition)
        Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

        Dim storyboardVariable As New Storyboard()
        storyboardVariable.Children.Add(anima)

        AddHandler storyboardVariable.Completed, Sub(sender As Object, args As EventArgs)
                                                     pageHostBaseIndex = (pageHostBaseIndex + 2) Mod 3
                                                     Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex

                                                     ' Transfer the indices from current to next
                                                     nextPageChapterIndex = Me.ChapterIndex
                                                     nextPagePageIndex = Me.PageIndex

                                                     ' Transfer the indices from previous to current
                                                     Me.ChapterIndex = prevPageChapterIndex
                                                     Me.PageIndex = prevPagePageIndex

                                                     ' Set up the new previous page
                                                     SetUpPreviousPage()
                                                     viewerTouchMode = viewerTouchMode.Reading
                                                 End Sub

        viewerTouchMode = ViewerTouchMode.Animating
        storyboardVariable.Begin()
    End Sub

    Private Sub AnimateToSamePage()
        Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex, .Duration = CalculateFractionalDuration(pageHostBaseIndex)}

        Storyboard.SetTarget(anima, Me.PageTransition)
        Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

        Dim storyboardVariable As New Storyboard()
        storyboardVariable.Children.Add(anima)

        AddHandler storyboardVariable.Completed, Sub(sender As Object, args As EventArgs) viewerTouchMode = viewerTouchMode.Reading

        viewerTouchMode = ViewerTouchMode.Animating
        storyboardVariable.Begin()
    End Sub

    Public Sub AnimatePinchAndPanUndo()
        ' This should have been done already
        matrixTransform.Matrix = transformGroup.Value

        ' Transfer matrix transform to scale & translate
        scaleTransform.ScaleX = matrixTransform.Matrix.M11
        scaleTransform.ScaleY = matrixTransform.Matrix.M22
        scaleTransform.CenterX = 0
        scaleTransform.CenterY = 0
        translateTransform.X = matrixTransform.Matrix.OffsetX
        translateTransform.Y = matrixTransform.Matrix.OffsetY
        matrixTransform.Matrix = Matrix.Identity

        Dim storyboard As New Storyboard()

        ' Scale X Animation
        Dim anima As DoubleAnimation = New DoubleAnimation With {.To = 1, .Duration = New Duration(pinchUndoAnimationDuration)}

        Storyboard.SetTarget(anima, scaleTransform)
        Storyboard.SetTargetProperty(anima, New PropertyPath(ScaleTransform.ScaleXProperty))
        storyboard.Children.Add(anima)

        ' Scale Y Animation
        anima = New DoubleAnimation With {.To = 1, .Duration = New Duration(pinchUndoAnimationDuration)}

        Storyboard.SetTarget(anima, scaleTransform)
        Storyboard.SetTargetProperty(anima, New PropertyPath(ScaleTransform.ScaleYProperty))
        storyboard.Children.Add(anima)

        ' Translate X Animation
        anima = New DoubleAnimation With {.To = 0, .Duration = New Duration(pinchUndoAnimationDuration)}

        Storyboard.SetTarget(anima, translateTransform)
        Storyboard.SetTargetProperty(anima, New PropertyPath(TranslateTransform.XProperty))
        storyboard.Children.Add(anima)

        ' Translate Y Animation
        anima = New DoubleAnimation With {.To = 0, .Duration = New Duration(pinchUndoAnimationDuration)}

        Storyboard.SetTarget(anima, translateTransform)
        Storyboard.SetTargetProperty(anima, New PropertyPath(TranslateTransform.YProperty))
        storyboard.Children.Add(anima)

        ' Set Completed handler
        AddHandler storyboard.Completed, Sub()
                                             viewerDisplayMode = ViewerDisplayMode.Normal
                                             viewerTouchMode = ViewerTouchMode.Reading
                                         End Sub

        ' Start the animation
        viewerTouchMode = ViewerTouchMode.Animating
        storyboard.Begin()
    End Sub

    Private Function CalculateFractionalDuration(ByVal fractionalBaseIndexDestination As Double) As Duration
        Dim fraction As Double = Math.Abs(fractionalBaseIndexDestination - Me.PageTransition.FractionalBaseIndex)
        Dim msec As Double = fraction * Me.PageTransition.AnimationDuration
        Dim timeSpan As TimeSpan = TimeSpan.FromMilliseconds(msec)
        Return New Duration(timeSpan)
    End Function

    Private Shared Sub OnPageProviderChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
    End Sub

    Private Shared Sub OnPageTransitionChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, BookViewer).OnPageTransitionChanged(CType(args.OldValue, PageTransition), CType(args.NewValue, PageTransition))
    End Sub

    Private Sub OnPageTransitionChanged(ByVal oldPageTransition As PageTransition, ByVal newPageTransition As PageTransition)
        Dim fractionalBaseIndex As Double = -1

        If oldPageTransition IsNot Nothing Then
            fractionalBaseIndex = oldPageTransition.FractionalBaseIndex
            oldPageTransition.Detach()
        End If
        If newPageTransition IsNot Nothing Then
            newPageTransition.Attach(LayoutRoot, pageContainer0, pageContainer1, pageContainer2)
            newPageTransition.FractionalBaseIndex = fractionalBaseIndex
        End If
    End Sub

    Shared Sub OnPageChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        Dim bookViewer As BookViewer = TryCast(obj, BookViewer)

        If bookViewer.PageChangedEvent IsNot Nothing Then
            bookViewer.PageChangedEvent(bookViewer, EventArgs.Empty)
        End If
    End Sub
End Class
