﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls


Partial Public Class BookmarkDialog
    Inherits UserControl
    Implements ITombstonable
    Private _bookmark As Bookmark

    Public Shared ReadOnly CaptionProperty As DependencyProperty = DependencyProperty.Register("Caption", GetType(String), GetType(BookmarkDialog), New PropertyMetadata(AddressOf OnCaptionPropertyChanged))

    Public Event DialogDismissed As EventHandler(Of CancelEventArgs)

    Public Sub New()
        InitializeComponent()

        AddHandler Loaded, Sub(sender, args) labelTextBox.Focus()
    End Sub

    Public Property Bookmark() As Bookmark
        Set(ByVal value As Bookmark)
            _bookmark = value
            labelTextBox.Text = _bookmark.Label
            labelTextBox.Select(labelTextBox.Text.Length, 0)
        End Set
        Get
            Return _bookmark
        End Get
    End Property

    Public Property Caption() As String
        Set(ByVal value As String)
            SetValue(CaptionProperty, value)
        End Set
        Get
            Return CStr(GetValue(CaptionProperty))
        End Get
    End Property

    Public Sub SaveState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.SaveState
        state(prefix & "bookmark") = Me.Bookmark
        state(prefix & "text") = labelTextBox.Text
        state(prefix & "selectionStart") = labelTextBox.SelectionStart
        state(prefix & "selectionLength") = labelTextBox.SelectionLength
    End Sub

    Public Sub RestoreState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.RestoreState
        Me.Bookmark = TryCast(state(prefix & "bookmark"), Bookmark)
        labelTextBox.Text = TryCast(state(prefix & "text"), String)
        labelTextBox.SelectionStart = CInt(Fix(state(prefix & "selectionStart")))
        labelTextBox.SelectionLength = CInt(Fix(state(prefix & "selectionLength")))
    End Sub

    Private Shared Sub OnCaptionPropertyChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, BookmarkDialog).captionTextBlock.Text = TryCast(args.NewValue, String)
    End Sub

    Private Sub OnCancelButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        RaiseEvent DialogDismissed(Me, New CancelEventArgs(True))
    End Sub

    Private Sub OnOkButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Me.Bookmark.Label = labelTextBox.Text

        RaiseEvent DialogDismissed(Me, New CancelEventArgs())
    End Sub

    Private Sub OnLabelTextBoxTextChanged(ByVal sender As Object, ByVal args As TextChangedEventArgs)
        okButton.IsEnabled = (TryCast(sender, TextBox)).Text.Length > 0
    End Sub
End Class
