﻿Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports System.Windows.Media


<TemplatePart(Name:="HorizontalTemplate", Type:=GetType(FrameworkElement)), TemplatePart(Name:="HorizontalTrack", Type:=GetType(Border)), TemplatePart(Name:="HorizontalThumb", Type:=GetType(ManipulationThumb)), TemplatePart(Name:="HorizontalThumbTranslate", Type:=GetType(TranslateTransform)), TemplatePart(Name:="VerticalTemplate", Type:=GetType(FrameworkElement)), TemplatePart(Name:="VerticalTrack", Type:=GetType(Border)), TemplatePart(Name:="VerticalThumb", Type:=GetType(ManipulationThumb)), TemplatePart(Name:="VerticalThumbTranslate", Type:=GetType(TranslateTransform))>
Public Class IntegralSlider
    Inherits Control
    Private horizontalTemplate, verticalTemplate As FrameworkElement
    Private horizontalTrack, verticalTrack, activeTrack As Border
    Private horizontalThumb, verticalThumb, activeThumb As ManipulationThumb
    Private horizontalThumbTranslate, verticalThumbTranslate, activeThumbTranslate As TranslateTransform
    Private isDragging As Boolean
    Private trackSize As Double
    Private rawOffset As Double

    Public Shared ReadOnly OrientationProperty As DependencyProperty = DependencyProperty.Register("Orientation", GetType(Orientation), GetType(IntegralSlider), New PropertyMetadata(Orientation.Horizontal, AddressOf OnOrientationChanged))

    Public Shared ReadOnly MinimumProperty As DependencyProperty = DependencyProperty.Register("Minimum", GetType(Integer), GetType(IntegralSlider), New PropertyMetadata(0, AddressOf OnMinMaxChanged))

    Public Shared ReadOnly MaximumProperty As DependencyProperty = DependencyProperty.Register("Maximum", GetType(Integer), GetType(IntegralSlider), New PropertyMetadata(10, AddressOf OnMinMaxChanged))

    Public Shared ReadOnly ValueProperty As DependencyProperty = DependencyProperty.Register("Value", GetType(Integer), GetType(IntegralSlider), New PropertyMetadata(0, AddressOf OnValueChanged))

    Public Event ValueChanged As RoutedPropertyChangedEventHandler(Of Integer)

    Public Sub New()
        Me.DefaultStyleKey = GetType(IntegralSlider)
        AddHandler Loaded, Sub(sender, args) MoveThumbToPosition()
    End Sub

    Public ReadOnly Property SmallThumbDimension() As Double
        Get
            Return 48
        End Get
    End Property
    Public ReadOnly Property LargeThumbDimension() As Double
        Get
            Return 96
        End Get
    End Property

    Public Property Orientation() As Orientation
        Set(ByVal value As Orientation)
            SetValue(OrientationProperty, value)
        End Set
        Get
            Return CType(GetValue(OrientationProperty), Orientation)
        End Get
    End Property

    Public Property Minimum() As Integer
        Set(ByVal value As Integer)
            SetValue(MinimumProperty, value)
        End Set
        Get
            Return CInt(Fix(GetValue(MinimumProperty)))
        End Get
    End Property

    Public Property Maximum() As Integer
        Set(ByVal value As Integer)
            SetValue(MaximumProperty, value)
        End Set
        Get
            Return CInt(Fix(GetValue(MaximumProperty)))
        End Get
    End Property

    Public Property Value() As Integer
        Set(ByVal value As Integer)
            SetValue(ValueProperty, value)
        End Set
        Get
            Return CInt(Fix(GetValue(ValueProperty)))
        End Get
    End Property

    Private Shared Sub OnOrientationChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, IntegralSlider).OnOrientationChanged(args)
    End Sub

    Private Sub OnOrientationChanged(ByVal args As DependencyPropertyChangedEventArgs)
        SwapOrientation()
    End Sub

    Private Shared Sub OnMinMaxChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, IntegralSlider).OnMinMaxChanged(args)
    End Sub

    Private Sub OnMinMaxChanged(ByVal args As DependencyPropertyChangedEventArgs)
        If (Value < Minimum AndAlso Minimum <= Maximum) OrElse (Value > Minimum AndAlso Minimum >= Maximum) Then
            Value = Minimum

        ElseIf (Value < Maximum AndAlso Maximum <= Minimum) OrElse (Value > Maximum AndAlso Maximum >= Minimum) Then
            Value = Maximum
        End If

        If Not isDragging Then
            MoveThumbToPosition()
        End If
    End Sub

    Private Shared Sub OnValueChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        TryCast(obj, IntegralSlider).OnValueChanged(CInt(Fix(args.OldValue)), CInt(Fix(args.NewValue)))
    End Sub

    Protected Overridable Sub OnValueChanged(ByVal oldValue As Integer, ByVal newValue As Integer)
        If (Minimum <= Maximum AndAlso Value >= Minimum AndAlso Value <= Maximum) OrElse (Maximum <= Minimum AndAlso Value >= Maximum AndAlso Value <= Minimum) Then
            RaiseEvent ValueChanged(Me, New RoutedPropertyChangedEventArgs(Of Integer)(oldValue, newValue))

            If Not isDragging Then
                MoveThumbToPosition()
            End If
        Else
            OnMinMaxChanged(New DependencyPropertyChangedEventArgs())
        End If
    End Sub

    Public Overrides Sub OnApplyTemplate()
        MyBase.OnApplyTemplate()

        horizontalTemplate = TryCast(Me.GetTemplateChild("HorizontalTemplate"), FrameworkElement)
        horizontalTrack = TryCast(Me.GetTemplateChild("HorizontalTrack"), Border)
        horizontalThumb = TryCast(Me.GetTemplateChild("HorizontalThumb"), ManipulationThumb)
        horizontalThumbTranslate = TryCast(Me.GetTemplateChild("HorizontalThumbTranslate"), TranslateTransform)

        verticalTemplate = TryCast(Me.GetTemplateChild("VerticalTemplate"), FrameworkElement)
        verticalTrack = TryCast(Me.GetTemplateChild("VerticalTrack"), Border)
        verticalThumb = TryCast(Me.GetTemplateChild("VerticalThumb"), ManipulationThumb)
        verticalThumbTranslate = TryCast(Me.GetTemplateChild("VerticalThumbTranslate"), TranslateTransform)

        SwapOrientation()
        MoveThumbToPosition()
    End Sub

    Private Sub SwapOrientation()
        If activeThumb IsNot Nothing AndAlso activeThumbTranslate IsNot Nothing Then
            RemoveHandler activeThumb.DragStarted, AddressOf OnThumbDragStarted
            RemoveHandler activeThumb.DragDelta, AddressOf OnThumbDragDelta
            RemoveHandler activeThumb.DragCompleted, AddressOf OnThumbDragCompleted
        End If

        If Orientation = Orientation.Horizontal Then
            If horizontalTemplate IsNot Nothing Then
                horizontalTemplate.Visibility = Visibility.Visible
            End If

            If verticalTemplate IsNot Nothing Then
                verticalTemplate.Visibility = Visibility.Collapsed
            End If

            activeTrack = horizontalTrack
            activeThumb = horizontalThumb
            activeThumbTranslate = horizontalThumbTranslate
        Else
            If horizontalTemplate IsNot Nothing Then
                horizontalTemplate.Visibility = Visibility.Collapsed
            End If

            If verticalTemplate IsNot Nothing Then
                verticalTemplate.Visibility = Visibility.Visible
            End If

            activeTrack = verticalTrack
            activeThumb = verticalThumb
            activeThumbTranslate = verticalThumbTranslate
        End If

        If activeThumb IsNot Nothing AndAlso activeThumbTranslate IsNot Nothing Then
            AddHandler activeThumb.DragStarted, AddressOf OnThumbDragStarted
            AddHandler activeThumb.DragDelta, AddressOf OnThumbDragDelta
            AddHandler activeThumb.DragCompleted, AddressOf OnThumbDragCompleted
        End If
        MyBase.OnApplyTemplate()
    End Sub

    Private Sub MoveThumbToPosition()
        If activeThumb Is Nothing OrElse activeThumbTranslate Is Nothing Then
            Return
        End If

        Dim trackSize As Double = GetTrackSize()
        Dim offset As Double = (CDbl(Value - Minimum) / Math.Abs(Maximum - Minimum)) * trackSize

        If Orientation = Orientation.Horizontal Then
            activeThumbTranslate.X = offset
        Else
            activeThumbTranslate.Y = offset
        End If
    End Sub

    Private Function GetTrackSize() As Double
        Dim container As FrameworkElement = If((TryCast(activeTrack, FrameworkElement)), (TryCast(Me, FrameworkElement)))

        If Orientation = Orientation.Horizontal Then
            Return container.ActualWidth - activeThumb.Width
        End If

        Return container.ActualHeight - activeThumb.Height
    End Function

    Private Sub OnThumbDragStarted(ByVal sender As Object, ByVal args As DragStartedEventArgs)
        isDragging = True

        Dim container As FrameworkElement = If((TryCast(activeTrack, FrameworkElement)), (TryCast(Me, FrameworkElement)))

        If Orientation = Orientation.Horizontal Then
            trackSize = container.ActualWidth - activeThumb.Width
            rawOffset = horizontalThumbTranslate.X
        Else
            trackSize = container.ActualHeight - activeThumb.Height
            rawOffset = verticalThumbTranslate.Y
        End If
    End Sub

    Private Sub OnThumbDragDelta(ByVal sender As Object, ByVal args As DragDeltaEventArgs)
        If Not isDragging Then
            Return
        End If

        If Orientation = Orientation.Horizontal Then
            rawOffset += args.HorizontalChange
        Else
            rawOffset += args.VerticalChange
        End If

        rawOffset = Math.Max(0, Math.Min(trackSize, rawOffset))
        Dim useOffset As Double = rawOffset - (trackSize / 35) * Math.Sin(2 * Math.PI * (rawOffset / trackSize) * Math.Abs(Maximum - Minimum))

        If Orientation = Orientation.Horizontal Then
            activeThumbTranslate.X = useOffset
        Else
            activeThumbTranslate.Y = useOffset
        End If

        Value = CInt(Fix(Math.Round(Minimum + useOffset / trackSize * (Maximum - Minimum))))
    End Sub

    Private Sub OnThumbDragCompleted(ByVal sender As Object, ByVal args As DragCompletedEventArgs)
        isDragging = False
    End Sub
End Class

