﻿Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Xml.Serialization

Namespace RdfDeserialization
    ' Project Gutenberg maintains a file http://www.gutenberg.org/ebooks/{id}.rdf for each book.
    ' I had not planned to use these files, since it seemed I could get sufficient stuff from
    '      the large catalog file, and I needed that anyway.
    ' However, for some books, e.g., 642, the UTF-8 file indicated in the catalog was empty,
    '      and a different filename appeared in the RDF file.
    ' Moreover, for other books, e.g., 

    Friend Class XmlNamespace
        Public Const cc As String = "http://web.resource.org/cc/"
        Public Const dcterms As String = "http://purl.org/dc/terms/"
        Public Const rdf As String = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        Public Const pgterms As String = "http://www.gutenberg.org/2009/pgterms/"
    End Class

    <XmlRoot(Namespace:=XmlNamespace.rdf)>
    Public Class RDF
        ' Valid for catalog and individual books
        <XmlElement(ElementName:="Description", Namespace:=XmlNamespace.rdf)>
        Public Property Description() As Description

        ' Valid for catalog only
        <XmlElement(ElementName:="etext", Namespace:=XmlNamespace.pgterms)>
        Public Property etext() As List(Of etext)

        ' Valid for individual books
        <XmlElement(ElementName:="Work", Namespace:=XmlNamespace.cc)>
        Public Property Work() As Work

        <XmlElement(ElementName:="ebook", Namespace:=XmlNamespace.pgterms)>
        Public Property ebook() As ebook

        <XmlElement(ElementName:="agent", Namespace:=XmlNamespace.pgterms)>
        Public Property agents() As List(Of agent)

        <XmlElement(ElementName:="file", Namespace:=XmlNamespace.pgterms)>
        Public Property Files() As List(Of file)

        Public Shared Function Deserialize(ByVal xml As String) As RDF
            Using reader As New StringReader(xml)
                Return RDF.Deserialize(reader)
            End Using
        End Function

        Public Shared Function Deserialize(ByVal reader As TextReader) As RDF
            Dim serializer As New XmlSerializer(GetType(RDF))
            Return TryCast(serializer.Deserialize(reader), RDF)
        End Function
    End Class

    Public Class Description
        <XmlElement(ElementName:="value", Namespace:=XmlNamespace.rdf)>
        Public Property Values() As List(Of value)
    End Class

    Public Class etext
        <XmlElement(ElementName:="publisher", Namespace:=XmlNamespace.dcterms)>
        Public Property publisher() As String

        <XmlElement(ElementName:="title", Namespace:=XmlNamespace.dcterms)>
        Public Property title() As String

        <XmlElement(ElementName:="creator", Namespace:=XmlNamespace.dcterms)>
        Public Property creator() As List(Of String)

        <XmlElement(ElementName:="friendlytitle", Namespace:=XmlNamespace.dcterms)>
        Public Property friendlytitle() As String
    End Class

    Public Class Work

    End Class

    Public Class ebook
        <XmlElement(ElementName:="title", Namespace:=XmlNamespace.dcterms)>
        Public Property title() As String

        <XmlElement(ElementName:="publisher", Namespace:=XmlNamespace.dcterms)>
        Public Property publisher() As String

        <XmlElement(ElementName:="issued", Namespace:=XmlNamespace.dcterms)>
        Public Property issued() As String

        <XmlElement(ElementName:="language", Namespace:=XmlNamespace.dcterms)>
        Public Property language() As String

        <XmlElement(ElementName:="type", Namespace:=XmlNamespace.dcterms)>
        Public Property type() As fileType
    End Class

    Public Class fileType
        <XmlElement(ElementName:="Description", Namespace:=XmlNamespace.rdf)>
        Public Property Description() As Description
    End Class

    Public Class agent
        <XmlElement(ElementName:="name", Namespace:=XmlNamespace.pgterms)>
        Public Property name() As String

        <XmlElement(ElementName:="birthdate", Namespace:=XmlNamespace.pgterms)>
        Public Property birthdate() As String

        <XmlElement(ElementName:="deathdate", Namespace:=XmlNamespace.pgterms)>
        Public Property deathdate() As String

        Public Overrides Function ToString() As String
            If Not String.IsNullOrEmpty(birthdate) Then
                If Not String.IsNullOrEmpty(deathdate) Then
                    Return String.Format("{0} ({1} - {2})", name, birthdate, deathdate)
                Else
                    Return String.Format("{0} (b. {1})", name, birthdate)
                End If
            End If

            Return name
        End Function
    End Class

    Public Class value
        <XmlText()>
        Public Property content() As String
    End Class

    Public Class file
        <XmlAttribute(Namespace:=XmlNamespace.pgterms)>
        Public Property about() As String

        <XmlElement(ElementName:="extent", Namespace:=XmlNamespace.dcterms)>
        Public Property extent() As String

        <XmlElement(ElementName:="format", Namespace:=XmlNamespace.dcterms)>
        Public Property format() As format
    End Class

    Public Class format
        <XmlElement(ElementName:="Description", Namespace:=XmlNamespace.rdf)>
        Public Property Description() As Description
    End Class
End Namespace