﻿Imports System
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Petzold.Phone.EBookReader


Partial Public Class AnnotationsPage
    Inherits PhoneApplicationPage
    Private book As BookInfo = (TryCast(Application.Current, App)).CurrentBook
    Private activePopup As Popup
    Private saveNote As String

    Public Sub New()
        InitializeComponent()
        Me.DataContext = book
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        If activePopup IsNot Nothing Then
            Me.State("popup") = activePopup.Name
            Me.State("popupOffsetX") = activePopup.HorizontalOffset
            Me.State("popupOffsetY") = activePopup.VerticalOffset
            Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

            If popupChild IsNot Nothing Then
                popupChild.SaveState(activePopup.Name, Me.State)
            End If
        ElseIf Me.State.ContainsKey("popup") Then
            Me.State.Remove("popup")
        End If

        MyBase.OnNavigatedFrom(args)
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        If activePopup Is Nothing AndAlso Me.State.ContainsKey("popup") Then
            activePopup = TryCast(Me.FindName(TryCast(Me.State("popup"), String)), Popup)
            activePopup.HorizontalOffset = CDbl(Me.State("popupOffsetX"))
            activePopup.VerticalOffset = CDbl(Me.State("popupOffsetY"))
            Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

            If popupChild IsNot Nothing Then
                popupChild.RestoreState(activePopup.Name, Me.State)
            End If

            annotationsListBox.IsEnabled = False
            activePopup.IsOpen = True
        End If

        MyBase.OnNavigatedTo(args)
    End Sub

    Protected Overrides Sub OnBackKeyPress(ByVal args As CancelEventArgs)
        If activePopup IsNot Nothing Then
            activePopup.IsOpen = False
            activePopup = Nothing
            annotationsListBox.IsEnabled = True
            args.Cancel = True
        End If

        MyBase.OnBackKeyPress(args)
    End Sub

    Private Sub OnAnnotationsListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
        Dim annotation As Annotation = TryCast(annotationsListBox.SelectedItem, Annotation)

        If annotation IsNot Nothing Then
            book.IsChapterAndPageInvalid = True
            book.CurrentCharacterOffset = annotation.CharacterOffset
            Me.NavigationService.GoBack()
        End If
    End Sub

    Private Sub OnEditButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim btn As Button = TryCast(sender, Button)
        Dim annotation As Annotation = TryCast(btn.Tag, Annotation)

        saveNote = annotation.Note
        annotationDialog.Annotation = annotation
        annotationsListBox.IsEnabled = False
        annotationDialogPopup.IsOpen = True
        activePopup = annotationDialogPopup
    End Sub

    Private Sub OnAnnotationDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
        Dim annotationDialog As AnnotationDialog = TryCast(sender, AnnotationDialog)

        ' See BookmarksPage.xaml.vb for the following logic.
        If args.Cancel Then
            annotationDialog.Annotation.Note = saveNote
        Else
            Dim index As Integer = book.Annotations.IndexOf(annotationDialog.Annotation)
            book.Annotations(index) = annotationDialog.Annotation
        End If

        annotationDialogPopup.IsOpen = False
        annotationsListBox.IsEnabled = True
        activePopup = Nothing
    End Sub

    Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim btn As Button = TryCast(sender, Button)
        Dim annotation As Annotation = TryCast(btn.Tag, Annotation)
        deleteConfirmDialog.Message = annotation.Note
        annotationsListBox.IsEnabled = False
        deleteConfirmDialog.Tag = annotation
        deleteConfirmDialogPopup.IsOpen = True
        activePopup = deleteConfirmDialogPopup
    End Sub

    Private Sub OnDeleteConfirmDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
        If Not args.Cancel Then
            Dim dlg As OkCancelDialog = TryCast(sender, OkCancelDialog)
            Dim annotation As Annotation = TryCast(dlg.Tag, Annotation)
            book.Annotations.Remove(annotation)
        End If

        deleteConfirmDialogPopup.IsOpen = False
        annotationsListBox.IsEnabled = True
        activePopup = Nothing
    End Sub
End Class
