﻿Imports System.Collections.Generic
Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Xml.Serialization
Imports Petzold.Phone.EBookReader

Partial Public Class App
    Inherits Application

    ' Field for saving and restoring app settings
    Private Const appSettingsFilename As String = "PhreeBookReader.settings"

    ' Properties for saving and restoring app & book settings
    Private _AppSettings As AppSettings
    Public Property AppSettings() As AppSettings
        Get
            Return _AppSettings
        End Get
        Private Set(ByVal value As AppSettings)
            _AppSettings = value
        End Set
    End Property
    Public Property CurrentBook() As BookInfo

    ' Property for miscelleneous settings
    Public Property TombstoneObjects() As IDictionary(Of String, Object)

    ''' <summary>
    ''' Provides easy access to the root frame of the Phone Application.
    ''' </summary>
    ''' <returns>The root frame of the Phone Application.</returns>
    Public Property RootFrame As PhoneApplicationFrame

    ''' <summary>
    ''' Constructor for the Application object.
    ''' </summary>
    Public Sub New()
        ' Standard Silverlight initialization
        InitializeComponent()

        ' Phone-specific initialization
        InitializePhoneApplication()

        ' Show graphics profiling information while debugging.
        If System.Diagnostics.Debugger.IsAttached Then
            ' Display the current frame rate counters.
            Application.Current.Host.Settings.EnableFrameRateCounter = True

            ' Show the areas of the app that are being redrawn in each frame.
            'Application.Current.Host.Settings.EnableRedrawRegions = True

            ' Enable non-production analysis visualization mode, 
            ' which shows areas of a page that are handed off to GPU with a colored overlay.
            'Application.Current.Host.Settings.EnableCacheVisualization = True


            ' Disable the application idle detection by setting the UserIdleDetectionMode property of the
            ' application's PhoneApplicationService object to Disabled.
            ' Caution:- Use this under debug mode only. Application that disables user idle detection will continue to run
            ' and consume battery power when the user is not using the phone.
            PhoneApplicationService.Current.UserIdleDetectionMode = IdleDetectionMode.Disabled
        End If
        ' Dump Isolated Storage is the debugger is attached
        If System.Diagnostics.Debugger.IsAttached Then
            Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
                DumpDirectories(isoStorage, "*.*", 0)
            End Using
        End If
    End Sub
    Private Sub DumpDirectories(ByVal isoStorage As IsolatedStorageFile, ByVal pattern As String, ByVal indent As Integer)
        DumpFiles(isoStorage, pattern, indent)

        Dim directoryNames() As String = Nothing

        Try
            directoryNames = isoStorage.GetDirectoryNames(pattern)
        Catch
            Return
        End Try

        Array.Sort(directoryNames)

        For Each directoryName In directoryNames
            System.Diagnostics.Debug.WriteLine(String.Format("{0}{1}", New String(" "c, 4 * indent), directoryName))
            DumpDirectories(isoStorage, Path.Combine(directoryName, pattern), indent + 1)
        Next directoryName
    End Sub

    Private Sub DumpFiles(ByVal isoStorage As IsolatedStorageFile, ByVal pattern As String, ByVal indent As Integer)
        Dim filenames() As String = Nothing

        Try
            filenames = isoStorage.GetFileNames(pattern)
        Catch
            Return
        End Try

        Array.Sort(filenames)

        For Each filename In filenames
            Using stream As IsolatedStorageFileStream = isoStorage.OpenFile(filename, FileMode.Open, FileAccess.Read, FileShare.Read)
                System.Diagnostics.Debug.WriteLine("{0}{1} - {2} bytes", New String(" "c, 4 * indent), filename, stream.Length)
            End Using
        Next filename
    End Sub
    ' Code to execute when the application is launching (eg, from Start)
    ' This code will not execute when the application is reactivated
    Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
        LoadSettings(False)
    End Sub

    ' Code to execute when the application is activated (brought to foreground)
    ' This code will not execute when the application is first launched
    Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
        LoadSettings(True)
    End Sub

    ' Code to execute when the application is deactivated (sent to background)
    ' This code will not execute when the application is closing
    Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
        SaveSettings(True)
    End Sub

    ' Code to execute when the application is closing (eg, user hit Back)
    ' This code will not execute when the application is deactivated
    Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
        SaveSettings(False)
    End Sub

    Private Sub LoadSettings(ByVal tombstoning As Boolean)
        ' Application Settings from isolated storage
        Me.AppSettings = AppSettings.Load(appSettingsFilename)

        ' Miscellaneous tombstone objects
        Me.TombstoneObjects = New Dictionary(Of String, Object)()

        If tombstoning Then
            Dim appService As PhoneApplicationService = PhoneApplicationService.Current

            ' Current book
            If appService.State.ContainsKey("currentBookID") Then
                Dim bookID As Integer = CInt(Fix(appService.State("currentBookID")))
                appService.State.Remove("currentBookID")
                Me.CurrentBook = BookInfo.Load(bookID)
            End If

            ' Transfer miscellaneous tombstone objects from ApplicationService state dictionary
            For Each key In appService.State.Keys
                If key.StartsWith("tombstoneObjects") Then
                    Me.TombstoneObjects.Add(key.Substring(16), appService.State(key))
                End If
            Next key

            ' Then remove them from ApplicationService state dictionary
            For Each key In Me.TombstoneObjects.Keys
                appService.State.Remove("tombstoneObjects" & key)
            Next key
        End If
    End Sub

    Private Sub SaveSettings(ByVal tombstoning As Boolean)
        ' Save application settings
        Me.AppSettings.Save(appSettingsFilename)

        If tombstoning Then
            Dim appService As PhoneApplicationService = PhoneApplicationService.Current

            ' Save current book information
            If Me.CurrentBook IsNot Nothing Then
                appService.State.Add("currentBookID", Me.CurrentBook.ID)
                Me.CurrentBook.Save()
            End If

            ' Save miscellaneous tombstone objects
            If Me.TombstoneObjects.Count > 0 Then
                For Each key In Me.TombstoneObjects.Keys
                    appService.State.Add("tombstoneObjects" & key, Me.TombstoneObjects(key))
                Next key
            End If
        End If
    End Sub
    ' Code to execute if a navigation fails
    Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
        If Diagnostics.Debugger.IsAttached Then
            ' A navigation has failed; break into the debugger
            Diagnostics.Debugger.Break()
        End If
    End Sub

    Public Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs) Handles Me.UnhandledException

        ' Show graphics profiling information while debugging.
        If Diagnostics.Debugger.IsAttached Then
            Diagnostics.Debugger.Break()
        Else
            e.Handled = True
            MessageBox.Show(e.ExceptionObject.Message & Environment.NewLine & e.ExceptionObject.StackTrace,
                            "Error", MessageBoxButton.OK)
        End If
    End Sub

#Region "Phone application initialization"
    ' Avoid double-initialization
    Private phoneApplicationInitialized As Boolean = False

    ' Do not add any additional code to this method
    Private Sub InitializePhoneApplication()
        If phoneApplicationInitialized Then
            Return
        End If

        ' Create the frame but don't set it as RootVisual yet; this allows the splash
        ' screen to remain active until the application is ready to render.
        RootFrame = New PhoneApplicationFrame()
        AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

        ' Handle navigation failures
        AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

        ' Ensure we don't initialize again
        phoneApplicationInitialized = True
    End Sub

    ' Do not add any additional code to this method
    Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
        ' Set the root visual to allow the application to render
        If RootVisual IsNot RootFrame Then
            RootVisual = RootFrame
        End If

        ' Remove this handler since it is no longer needed
        RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
    End Sub
#End Region

End Class