﻿Imports System
Imports System.Windows
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Petzold.Phone.EBookReader
Imports PhreeBookReader.CatalogServiceReference


Partial Public Class BookInformationPage
    Inherits PhoneApplicationPage
    Private bookInfo As BookInfo
    Private gutenbergBook As GutenbergBook

    Public Sub New()
        InitializeComponent()
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
        Dim bookID As Integer = Int32.Parse(Me.NavigationContext.QueryString("bookid"))
        gutenbergBook = GutenbergBookExtensions.Load(bookID)
        catalogInfo.GutenbergBook = gutenbergBook

        bookInfo = BookInfo.Load(bookID)
        LayoutRoot.DataContext = bookInfo

        ' This is the case if invoked from BookViewerPage
        If Me.NavigationContext.QueryString.ContainsKey("doNotAllowDeletion") Then
            deleteButton.Visibility = Visibility.Collapsed
        End If

        MyBase.OnNavigatedTo(e)
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
        ' bookInfo will be null if the book has been deleted
        If bookInfo IsNot Nothing Then
            bookInfo.Save()

            ' Also, replace BookInfo in Bedside collection
            Dim appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
            If appSettings.Bedside.Contains(bookInfo) Then
                Dim index As Integer = appSettings.Bedside.IndexOf(bookInfo)
                appSettings.Bedside(index) = bookInfo
            End If
        End If

        MyBase.OnNavigatedFrom(e)
    End Sub

    Private Sub OnChangeDisplayTitleClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim uri As String = String.Format("/DisplayTitlePage.xaml?bookid={0}", bookInfo.ID)
        Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
    End Sub

    Private Sub OnChapterBreaksClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim uri As String = String.Format("/ChapterAnalysisPage.xaml?bookid={0}", bookInfo.ID)
        Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
    End Sub

    Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        optionsButtons.IsEnabled = False
        deleteConfirmationPanel.Visibility = Visibility.Visible
        scrollViewer.ScrollToBottom()
    End Sub

    Private Sub OnOkDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim deleteBookID As Integer = bookInfo.ID

        ' Delete the book, info file, and Gutenberg catalog file from isolated storage
        gutenbergBook.Delete(deleteBookID)
        bookInfo.Delete()

        ' Delete the book from the library
        Dim appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
        Dim library As Library = appSettings.Library
        Dim foundID As Boolean = False

        Dim iAuthor As Integer = 0
        Do While iAuthor < library.Authors.Count
            Dim author As AuthorInfo = library.Authors(iAuthor)

            Dim iTitle As Integer = 0
            Do While iTitle < author.Titles.Count
                Dim title As TitleInfo = author.Titles(iTitle)

                If deleteBookID = title.ID Then
                    author.Titles.RemoveAt(iTitle)

                    If author.Titles.Count = 0 Then
                        library.Authors.RemoveAt(iAuthor)
                    End If

                    foundID = True
                    Exit Do
                End If
                iTitle += 1
            Loop
            If foundID Then
                Exit Do
            End If
            iAuthor += 1
        Loop

        ' Delete the book from the bedside
        Dim iBook As Integer = 0
        Do While iBook < appSettings.Bedside.Count
            Dim book As BookInfo = appSettings.Bedside(iBook)

            If deleteBookID = book.ID Then
                appSettings.Bedside.RemoveAt(iBook)
                Exit Do
            End If
            iBook += 1
        Loop

        ' Set the field to null and similarly in App
        bookInfo = Nothing

        ' Go back to the main page
        Me.NavigationService.GoBack()
    End Sub

    Private Sub OnCancelDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        deleteConfirmationPanel.Visibility = Visibility.Collapsed
        optionsButtons.IsEnabled = True
    End Sub
End Class
