﻿Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports System.Windows.Media
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Microsoft.Phone.Tasks
Imports Petzold.Phone.EBookReader

    Partial Public Class BookViewerPage
        Inherits PhoneApplicationPage
        Private appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
        Private webBrowserTask As New WebBrowserTask()
        Private searchTask As New SearchTask()
        Private book As BookInfo
        Private pageProvider As PageProvider
        Private activePopup As Popup

        ' Three fields consolidated for tombstoning purposes
        Public Structure FindParams
            Public Property FirstFindFromTop() As Boolean
            Public Property ParagraphIndex() As Integer
            Public Property CharacterIndex() As Integer
        End Structure
    Private findParamsVariable As FindParams

        ' Also for tombstoning purposes
        Private currentHighlight As FindInfo

        Public Sub New()
            InitializeComponent()

            ' Set the fields of the appbar buttons and menu
            appbar = TryCast(Me.ApplicationBar, ApplicationBar)

            chaptersIconButton = TryCast(appbar.Buttons(0), ApplicationBarIconButton)
            bookmarksIconButton = TryCast(appbar.Buttons(1), ApplicationBarIconButton)
            annotationsIconButton = TryCast(appbar.Buttons(2), ApplicationBarIconButton)
            findIconButton = TryCast(appbar.Buttons(3), ApplicationBarIconButton)

            smallerTextSize = TryCast(Me.ApplicationBar.MenuItems(1), ApplicationBarMenuItem)
            largerTextSize = TryCast(Me.ApplicationBar.MenuItems(2), ApplicationBarMenuItem)

            AddHandler Loaded, AddressOf OnLoaded
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
            If activePopup IsNot Nothing Then
                Me.State("popup") = activePopup.Name
                Me.State("popupOffsetX") = activePopup.HorizontalOffset
                Me.State("popupOffsetY") = activePopup.VerticalOffset
            Me.State("findParams") = findParamsVariable

                If currentHighlight IsNot Nothing Then
                    Me.State("currentHighlight") = currentHighlight
                ElseIf Me.State.ContainsKey("currentHighlight") Then
                    Me.State.Remove("currentHighlight")
                End If

                Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

                If popupChild IsNot Nothing Then
                    popupChild.SaveState(activePopup.Name, Me.State)
                End If
            ElseIf Me.State.ContainsKey("popup") Then
                Me.State.Remove("popup")
            End If

            ' Save the date and time this book was last being read
            book.LastRead = Date.Now

            MyBase.OnNavigatedFrom(args)
        End Sub

        Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
            ' Only reload the book and recreate the PageProvider if there's really
            '  been some tombstoning, and not when navigating to another page
            '  in the app.
            If pageProvider Is Nothing Then
                book = (TryCast(Application.Current, App)).CurrentBook
                pageProvider = New PageProvider(book)
                bookViewer.PageProvider = pageProvider
                Me.DataContext = book
            End If
            If activePopup Is Nothing AndAlso Me.State.ContainsKey("popup") Then
                EnableViewerAndAppBar(False)
                activePopup = TryCast(Me.FindName(TryCast(Me.State("popup"), String)), Popup)
                activePopup.HorizontalOffset = CDbl(Me.State("popupOffsetX"))
                activePopup.VerticalOffset = CDbl(Me.State("popupOffsetY"))
            findParamsVariable = CType(Me.State("findParams"), FindParams)
                Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

                If Me.State.ContainsKey("currentHighlight") Then
                    currentHighlight = CType(Me.State("currentHighlight"), FindInfo)
                End If

                If popupChild IsNot Nothing Then
                    popupChild.RestoreState(activePopup.Name, Me.State)
                End If

                activePopup.IsOpen = True
            Else
                EnableViewerAndAppBar(True)
            End If

            ' Save the date and time this book was last being read (just in case)
            book.LastRead = Date.Now

            ' Elevate the book to the top of the Bedside list
            If appSettings.Bedside.Contains(book) Then
                appSettings.Bedside.Remove(book)
            End If
            appSettings.Bedside.Insert(0, book)

            ' Restrict the list to BesideMaxCount books
            Do While appSettings.Bedside.Count > AppSettings.BedsideMaxCount
                appSettings.Bedside.RemoveAt(AppSettings.BedsideMaxCount)
            Loop

            MyBase.OnNavigatedTo(args)
        End Sub

        ' Occurs after OnNavigatedTo
        Private Sub OnLoaded(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Select Case appSettings.PageTransition
                Case "slide"
                    bookViewer.PageTransition = New SlideTransition()
                Case "skew"
                    bookViewer.PageTransition = New SkewTransition()
                Case "curl"
                    bookViewer.PageTransition = New CurlTransition()
                Case "flip"
                    bookViewer.PageTransition = New FlipTransition()
            End Select

            ResetBookViewerInformationAndGotoPage()

            If currentHighlight IsNot Nothing Then
                bookViewer.HighlightText(currentHighlight)
            End If
        End Sub

        Protected Overrides Sub OnBackKeyPress(ByVal args As CancelEventArgs)
            If activePopup IsNot Nothing Then
                activePopup.IsOpen = False
                activePopup = Nothing
                EnableViewerAndAppBar(True)
                args.Cancel = True
            End If

            MyBase.OnBackKeyPress(args)
        End Sub

        Private Sub OnBookViewerPageChanged(ByVal sender As Object, ByVal args As EventArgs)
            Dim chapterIndex As Integer = bookViewer.ChapterIndex
            Dim pageIndex As Integer = bookViewer.PageIndex

            If chapterIndex < book.Chapters.Count AndAlso pageIndex < book.Chapters(chapterIndex).Pages.Count Then
                Dim accumulatedCharacterCount As Integer = book.Chapters(chapterIndex).Pages(pageIndex).AccumulatedCharacterCount

                ' Save for isolated storage settings
                book.CurrentChapterIndex = chapterIndex
                book.CurrentPageIndex = pageIndex
                book.CurrentCharacterOffset = accumulatedCharacterCount

                ' Save the date and time this book was last being read (just in case)
                book.LastRead = Date.Now

                ' Update the percentage indicator
                percentageText.Text = String.Format("{0:F1}%", book.Percentage)
            End If
        End Sub

        Private Sub ResetBookViewerInformationAndGotoPage()
            ' Set the BookViewer font information
            bookViewer.FontFamily = New FontFamily(appSettings.FontFamily)
            bookViewer.FontSize = CDbl(Me.Resources("PhoneFontSizeNormal")) * appSettings.FontSizeMultiplier

            ' Set the pageProvider font information; could cause purging of pagination info
            pageProvider.SetFont(bookViewer.FontFamily, bookViewer.FontSize)

            ' This is also a good place to enable the items on the menu
            largerTextSize.IsEnabled = appSettings.FontSizeMultiplier < AppSettings.MaximumFontSizeMultiplier
            smallerTextSize.IsEnabled = appSettings.FontSizeMultiplier > AppSettings.MinimumFontSizeMultiplier

            ' If page is expanded, animate it down
            bookViewer.AnimatePinchAndPanUndo()

            If book.IsChapterAndPageInvalid Then
                bookViewer.JumpToCharacterOffset(book.CurrentCharacterOffset)
                book.IsChapterAndPageInvalid = False ' set here or elsewhere????

                ' Depending on retention of pagination information, one or the other of these will go to the page
            ElseIf book.Chapters(book.CurrentChapterIndex).Pages.Count > book.CurrentPageIndex Then
                If Not bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex) Then
                    bookViewer.JumpToCharacterOffset(book.CurrentCharacterOffset)
                End If
            Else
                bookViewer.JumpToCharacterOffset(book.CurrentCharacterOffset)
            End If
        End Sub

        Private Sub OnAppBarChaptersButtonClick(ByVal sender As Object, ByVal args As EventArgs)
            ' Navigate to the Chapters page
            Me.NavigationService.Navigate(New Uri("/ChaptersPage.xaml", UriKind.Relative))
        End Sub

        Private Sub OnAppBarBookmarksButtonClick(ByVal sender As Object, ByVal args As EventArgs)
            ' Navigate to the Bookmarks page
            Me.NavigationService.Navigate(New Uri("/BookmarksPage.xaml", UriKind.Relative))
        End Sub

        Private Sub OnAppBarAnnotationsButtonClick(ByVal sender As Object, ByVal args As EventArgs)
            ' Navigate to the Annotations page
            Me.NavigationService.Navigate(New Uri("/AnnotationsPage.xaml", UriKind.Relative))
        End Sub

        Private Sub OnAppBarSearchButtonClick(ByVal sender As Object, ByVal args As EventArgs)
            bookViewer.AnimatePinchAndPanUndo()

            Dim chapter As ChapterInfo = book.Chapters(book.CurrentChapterIndex)
            Dim page As PageInfo = chapter.Pages(book.CurrentPageIndex)
        findParamsVariable.ParagraphIndex = page.ParagraphIndex
        findParamsVariable.CharacterIndex = page.CharacterIndex
        findParamsVariable.FirstFindFromTop = True

            EnableViewerAndAppBar(False)
            findDialogPopup.IsOpen = True
            findDialogPopup.VerticalOffset = 0
            activePopup = findDialogPopup
        End Sub

        Private Sub OnAppBarSettingsItemClick(ByVal sender As Object, ByVal args As EventArgs)
            ' Navigate to Settings page
            Me.NavigationService.Navigate(New Uri("/SettingsPage.xaml", UriKind.Relative))
        End Sub

        Private Sub OnAppBarTextSizeItemClick(ByVal sender As Object, ByVal args As EventArgs)
            If sender Is smallerTextSize Then
                appSettings.FontSizeMultiplier = Math.Max(AppSettings.MinimumFontSizeMultiplier, appSettings.FontSizeMultiplier - 0.1)
            ElseIf sender Is largerTextSize Then
                appSettings.FontSizeMultiplier = Math.Min(AppSettings.MaximumFontSizeMultiplier, appSettings.FontSizeMultiplier + 0.1)
            End If

            ResetBookViewerInformationAndGotoPage()
        End Sub

        Private Sub OnAppBarBookInformationClick(ByVal sender As Object, ByVal args As EventArgs)
            ' Save the date and time this book was last being read
            book.LastRead = Date.Now

            ' Save the BookInfo object
            book.Save()

            ' Navigate to information page
            Dim uri As String = String.Format("/BookInformationPage.xaml?bookid={0}&doNotAllowDeletion={1}", book.ID, 1)
            Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
        End Sub

        ' When a TextSelected event comes from the BookViewer control,
        '  disable the control and display the text-selection menu.
        Private Sub OnBookViewerTextSelected(ByVal sender As Object, ByVal args As TextSelectedEventArgs)
            ' Save highlight for tombstoning purposes
            currentHighlight = New FindInfo With {.ParagraphIndex = args.ParagraphIndex, .StartCharacterIndex = args.CharacterIndex, .EndCharacterIndex = args.CharacterIndex + args.CharacterCount}

            EnableViewerAndAppBar(False)

            textSelectionMenu.Tag = args
            textSelectionMenu.SelectedItem = Nothing
            textSelectionMenuPopup.IsOpen = True
            activePopup = textSelectionMenuPopup

            ' Position the menu
            textSelectionMenu.Measure(New Size(Double.PositiveInfinity, Double.PositiveInfinity))
            Dim selectionMenuSize As Size = textSelectionMenu.DesiredSize
            Dim offsetX As Double = 0
            Dim offsetY As Double = 0

            ' Try centering it underneath the text
            If args.Bounds.Bottom + selectionMenuSize.Height < ContentPanel.ActualHeight Then
                offsetX = (args.Bounds.Left + args.Bounds.Right - selectionMenuSize.Width) / 2
                offsetY = args.Bounds.Bottom
                ' Now try centering it above the text
            ElseIf args.Bounds.Top - selectionMenuSize.Height > 0 Then
                offsetX = (args.Bounds.Left + args.Bounds.Right - selectionMenuSize.Width) / 2
                offsetY = args.Bounds.Top - selectionMenuSize.Height
                ' How about at the right?
            ElseIf args.Bounds.Right + selectionMenuSize.Width < ContentPanel.ActualWidth Then
                offsetX = args.Bounds.Right
                offsetY = (args.Bounds.Top + args.Bounds.Bottom - selectionMenuSize.Height) / 2
                ' Maybe at the left will be OK
            ElseIf args.Bounds.Left - selectionMenuSize.Width > 0 Then
                offsetX = args.Bounds.Left - selectionMenuSize.Width
                offsetY = (args.Bounds.Top + args.Bounds.Bottom - selectionMenuSize.Height) / 2
                ' I guess we have to stick it in a corner
            Else
                If args.Bounds.Left > ContentPanel.ActualWidth - args.Bounds.Right Then
                    offsetX = 0
                Else
                    offsetX = ContentPanel.ActualWidth - selectionMenuSize.Width
                End If

                If args.Bounds.Top > ContentPanel.ActualHeight - args.Bounds.Bottom Then
                    offsetY = 0
                Else
                    offsetY = ContentPanel.ActualHeight - selectionMenuSize.Height
                End If
            End If

            ' Finally, make sure it's on the screen
            offsetX = Math.Max(0, Math.Min(offsetX, ContentPanel.ActualWidth - selectionMenuSize.Width))
            offsetY = Math.Max(0, Math.Min(offsetY, ContentPanel.ActualHeight - selectionMenuSize.Height))

            textSelectionMenuPopup.HorizontalOffset = offsetX
            textSelectionMenuPopup.VerticalOffset = offsetY
        End Sub

        Private Sub OnTextSelectionMenuSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
            Dim lstbox As ListBox = TryCast(sender, ListBox)
            Dim selectionArgs As TextSelectedEventArgs = TryCast(lstbox.Tag, TextSelectedEventArgs)

            If lstbox.SelectedItem IsNot Nothing Then
                textSelectionMenuPopup.IsOpen = False
                activePopup = Nothing
                EnableViewerAndAppBar(True)
                Dim command As String = TryCast((TryCast(lstbox.SelectedItem, ListBoxItem)).Content, String)
                Dim text As String = selectionArgs.Text

                Select Case command
                    Case "note"
                        Dim characterOffset As Integer = book.Paragraphs(selectionArgs.ParagraphIndex).AccumulatedCharacterCount + selectionArgs.CharacterIndex

                        Dim annotation As Annotation = New Annotation With {.Text = selectionArgs.Text, .Note = "", .ChapterTitle = book.Chapters(book.CurrentChapterIndex).Title, .CharacterOffset = characterOffset, .Percentage = 100.0 * characterOffset / book.TotalCharacterCount, .DateTime = Date.Now}
                        annotationDialog.Annotation = annotation
                        annotationDialogPopup.IsOpen = True
                        activePopup = annotationDialogPopup
                        EnableViewerAndAppBar(False)

                    Case "bing"
                        searchTask.SearchQuery = StripPunctuation(text)
                        searchTask.Show()

                    Case "dictionary"
                        ' "p" == "printer friendly" (no mobile version as of 8/15/11)
                        InvokeWebBrowser("http://www.thefreedictionary.com/p/{0}", text)

                    Case "wikipedia"
                        ' English mobile version
                        InvokeWebBrowser("http://en.m.wikipedia.org/wiki/{0}", text)

                    Case "cancel"
                End Select
            End If
        End Sub

        Private Sub InvokeWebBrowser(ByVal format As String, ByVal text As String)
            Dim url As String = String.Format(format, StripPunctuation(text))
            webBrowserTask.URL = Uri.EscapeDataString(url)
            webBrowserTask.Show()
        End Sub

        Private Function StripPunctuation(ByVal text As String) As String
            Dim strBuilder As New StringBuilder()

        For Each ch In text
            If (Not Char.IsPunctuation(ch)) OrElse ch = "-"c Then
                strBuilder.Append(ch)
            End If
        Next ch

            Return strBuilder.ToString()
        End Function

        Private Sub OnAnnotationDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
            Dim annotationDialog As AnnotationDialog = TryCast(sender, AnnotationDialog)

            If Not args.Cancel Then
                book.Annotations.Add(annotationDialog.Annotation)
            End If

            annotationDialogPopup.IsOpen = False
            activePopup = Nothing
            EnableViewerAndAppBar(True)
        End Sub

        Private Sub OnBookViewerBookmarkInserted(ByVal sender As Object, ByVal args As EventArgs)
            Dim bookmark As Bookmark = New Bookmark With {.Label = "bookmark" & (book.Bookmarks.Count + 1), .ChapterTitle = book.Chapters(book.CurrentChapterIndex).Title, .CharacterOffset = book.CurrentCharacterOffset, .Percentage = book.Percentage, .DateTime = Date.Now}

            bookmarkDialog.Bookmark = bookmark
            EnableViewerAndAppBar(False)
            bookmarkDialogPopup.IsOpen = True
            activePopup = bookmarkDialogPopup
        End Sub

        Private Sub OnBookmarkDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
            Dim bookmarkDialog As BookmarkDialog = TryCast(sender, BookmarkDialog)

            If Not args.Cancel Then
                book.Bookmarks.Add(bookmarkDialog.Bookmark)
            End If

            bookmarkDialogPopup.IsOpen = False
            activePopup = Nothing
            EnableViewerAndAppBar(True)
        End Sub

        Private Sub OnFindDialogFindText(ByVal sender As Object, ByVal args As FindTextEventArgs)
            ' Avoid refinding the previous find
        If (Not findParamsVariable.FirstFindFromTop) AndAlso (Not args.GoUp) Then
            findParamsVariable.CharacterIndex += 1
        End If

        Dim findInfo As FindInfo = pageProvider.FindText(findParamsVariable.ParagraphIndex, findParamsVariable.CharacterIndex, args.Text, args.MatchCase, args.WholeWord, args.GoUp)
            bookViewer.ClearHighlighting()
            currentHighlight = Nothing

            If findInfo Is Nothing Then
            Dim findDialogVaraible As FindDialog = TryCast(sender, FindDialog)
            findDialogVaraible.ShowNoMoreFinds()
            Else
                bookViewer.JumpToCharacterOffset(findInfo.CharacterOffset)
                bookViewer.HighlightText(findInfo)
                currentHighlight = findInfo

                If findInfo.WordLocation < findDialog.ActualHeight Then
                    findDialogPopup.VerticalOffset = bookViewer.ActualHeight - findDialog.ActualHeight - findDialog.Margin.Bottom - findDialog.Margin.Top
                Else
                    findDialogPopup.VerticalOffset = 0
                End If

            findParamsVariable.ParagraphIndex = findInfo.ParagraphIndex
            findParamsVariable.CharacterIndex = findInfo.StartCharacterIndex
            End If

        findParamsVariable.FirstFindFromTop = False
        End Sub

        Private Sub OnFindDialogDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
            findDialogPopup.IsOpen = False
            activePopup = Nothing
            EnableViewerAndAppBar(True)
        End Sub

        Private Sub EnableViewerAndAppBar(ByVal isEnabled As Boolean)
            ' On transition from disabled BookViewer to enabled BookViewer, clear highlighting
            If isEnabled AndAlso (Not bookViewer.IsHitTestVisible) Then
                bookViewer.ClearHighlighting()
                currentHighlight = Nothing
            End If

            bookViewer.IsHitTestVisible = isEnabled
            chaptersIconButton.IsEnabled = isEnabled
            bookmarksIconButton.IsEnabled = isEnabled AndAlso book.Bookmarks.Count > 0
            annotationsIconButton.IsEnabled = isEnabled AndAlso book.Annotations.Count > 0
            findIconButton.IsEnabled = isEnabled
            appbar.IsMenuEnabled = isEnabled
        End Sub
    End Class
