﻿Imports System
Imports System.ComponentModel
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Petzold.Phone.EBookReader

    Partial Public Class BookmarksPage
        Inherits PhoneApplicationPage
        Private book As BookInfo = (TryCast(Application.Current, App)).CurrentBook
        Private activePopup As Popup
        Private saveLabel As String

        Public Sub New()
            InitializeComponent()
            Me.DataContext = book
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
            If activePopup IsNot Nothing Then
                Me.State("popup") = activePopup.Name
                Me.State("popupOffsetX") = activePopup.HorizontalOffset
                Me.State("popupOffsetY") = activePopup.VerticalOffset
                Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

                If popupChild IsNot Nothing Then
                    popupChild.SaveState(activePopup.Name, Me.State)
                End If
            ElseIf Me.State.ContainsKey("popup") Then
                Me.State.Remove("popup")
            End If

            MyBase.OnNavigatedFrom(args)
        End Sub

        Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
            If activePopup Is Nothing AndAlso Me.State.ContainsKey("popup") Then
                activePopup = TryCast(Me.FindName(TryCast(Me.State("popup"), String)), Popup)
                activePopup.HorizontalOffset = CDbl(Me.State("popupOffsetX"))
                activePopup.VerticalOffset = CDbl(Me.State("popupOffsetY"))
                Dim popupChild As ITombstonable = TryCast(activePopup.Child, ITombstonable)

                If popupChild IsNot Nothing Then
                    popupChild.RestoreState(activePopup.Name, Me.State)
                End If

                bookmarksListBox.IsEnabled = False
                activePopup.IsOpen = True
            End If

            MyBase.OnNavigatedTo(args)
        End Sub

        Protected Overrides Sub OnBackKeyPress(ByVal args As CancelEventArgs)
            If activePopup IsNot Nothing Then
                activePopup.IsOpen = False
                activePopup = Nothing
                bookmarksListBox.IsEnabled = True
                args.Cancel = True
            End If

            MyBase.OnBackKeyPress(args)
        End Sub

        Private Sub OnBookmarksListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
            Dim bookmark As Bookmark = TryCast(bookmarksListBox.SelectedItem, Bookmark)

            If bookmark IsNot Nothing Then
                book.IsChapterAndPageInvalid = True
                book.CurrentCharacterOffset = bookmark.CharacterOffset
                Me.NavigationService.GoBack()
            End If
        End Sub

        Private Sub OnEditButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Dim btn As Button = TryCast(sender, Button)
            Dim bookmark As Bookmark = TryCast(btn.Tag, Bookmark)

            saveLabel = bookmark.Label
            bookmarkDialog.Bookmark = bookmark
            bookmarksListBox.IsEnabled = False
            bookmarkDialogPopup.IsOpen = True
            activePopup = bookmarkDialogPopup
        End Sub

        Private Sub OnBookmarkDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
            Dim bookmarkDialog As BookmarkDialog = TryCast(sender, BookmarkDialog)

            ' If the page has been tombstoned, the Bookmark object from 
            '  the dialog box is not the same object that was set to it in
            '  the OnEditButtonClick method, and not the same as the 
            '  corresponding object in the collection.

            ' Fortunately, Bookmark implements IEquatable<T>, so we can 
            '  find the matching Bookmark object in the Bookmarks collection.

            If args.Cancel Then
                ' This is good if the page hasn't been tombstoned;
                '  the saved label is restored to the single Bookmark object.
                ' If the page has been tombstoned, nothing much happens
                bookmarkDialog.Bookmark.Label = saveLabel
            Else
                ' If the page hasn't been tombstoned, nothing needs to
                '  be done here. But let's do the tombstoning logic to
                '  get the new Bookmark in the collection.
                Dim index As Integer = book.Bookmarks.IndexOf(bookmarkDialog.Bookmark)
                book.Bookmarks(index) = bookmarkDialog.Bookmark
            End If

            bookmarkDialogPopup.IsOpen = False
            bookmarksListBox.IsEnabled = True
            activePopup = Nothing
        End Sub

        Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Dim btn As Button = TryCast(sender, Button)
            Dim bookmark As Bookmark = TryCast(btn.Tag, Bookmark)
            deleteConfirmDialog.Message = bookmark.Label
            bookmarksListBox.IsEnabled = False
            deleteConfirmDialog.Tag = bookmark
            deleteConfirmDialogPopup.IsOpen = True
            activePopup = deleteConfirmDialogPopup
        End Sub

        Private Sub OnDeleteConfirmDialogDismissed(ByVal sender As Object, ByVal args As CancelEventArgs)
            If Not args.Cancel Then
                Dim dlg As OkCancelDialog = TryCast(sender, OkCancelDialog)
                Dim bookmark As Bookmark = TryCast(dlg.Tag, Bookmark)
                book.Bookmarks.Remove(bookmark)
            End If

            deleteConfirmDialogPopup.IsOpen = False
            bookmarksListBox.IsEnabled = True
            activePopup = Nothing
        End Sub
    End Class
