﻿Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Petzold.Phone.EBookReader


Partial Public Class ChapterAnalysisPage
    Inherits PhoneApplicationPage
    Private bookInfo As BookInfo
    Private anyChanges As Boolean = False

    Public Sub New()
        InitializeComponent()
    End Sub

    ' Load BookInfo from file
    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        Dim bookID As Integer = Int32.Parse(Me.NavigationContext.QueryString("bookid"))
        bookInfo = BookInfo.Load(bookID)

        chaptersItemsControl.ItemsSource = bookInfo.Chapters
        UpdateTextAndButtons()

        MyBase.OnNavigatedTo(args)
    End Sub

    ' Save altered BookInfo to file (nothing more needs to be tombstoned)
    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        If anyChanges Then
            bookInfo.IsChapterAndPageInvalid = True
            bookInfo.UpdateBookmarksAndAnnotations()
            bookInfo.Save()
        End If
        MyBase.OnNavigatedFrom(args)
    End Sub

    Private Sub UpdateTextAndButtons()
        numBlanks.Text = bookInfo.BlankLinesBetweenChapters.ToString()
        numChapters.Text = (bookInfo.Chapters.Count - 1).ToString()

        fewerButton.IsEnabled = bookInfo.BlankLinesBetweenChapters < 6
        moreButton.IsEnabled = bookInfo.BlankLinesBetweenChapters > 2
    End Sub

    Private Sub OnFewerButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        bookInfo.BlankLinesBetweenChapters += 1
        bookInfo.GenerateChapters()
        UpdateTextAndButtons()
        anyChanges = True
    End Sub

    Private Sub OnMoreButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        bookInfo.BlankLinesBetweenChapters -= 1
        bookInfo.GenerateChapters()
        UpdateTextAndButtons()
        anyChanges = True
    End Sub

    Private Sub OnResetButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        bookInfo.GenerateChapters()
        UpdateTextAndButtons()
        resetButton.IsEnabled = False
        anyChanges = True
    End Sub

    Private Sub OnDeleteButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Get the chapter to be deleted
        Dim chapter As ChapterInfo = TryCast((TryCast(sender, Button)).DataContext, ChapterInfo)
        Dim index As Integer = bookInfo.Chapters.IndexOf(chapter)

        ' Add it's paragraph count to the count of the preceding chapter
        bookInfo.Chapters(index - 1).ParagraphCount += chapter.ParagraphCount

        ' Remove that chapter from the collection
        bookInfo.Chapters.Remove(chapter)

        ' Update buttons, etc.
        UpdateTextAndButtons()
        resetButton.IsEnabled = True
        anyChanges = True
    End Sub
End Class
