﻿Imports System
Imports System.IO.IsolatedStorage
Imports System.Net
Imports System.Windows
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Petzold.Phone.EBookReader
Imports Petzold.Phone.EBookReader.RdfDeserialization
Imports PhreeBookReader.CatalogServiceReference

    Partial Public Class DownloadBookPage
        Inherits PhoneApplicationPage
        Private appService As PhoneApplicationService = PhoneApplicationService.Current
        Private appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings

        Private gutenbergBook As GutenbergBook
        Private bookInfo As BookInfo
        Private webClient As New WebClient()

        Private Enum DownloadMode
            RdfFile
            BookFile
        End Enum
    Private _downloadMode As DownloadMode

        Public Sub New()
            InitializeComponent()

            AddHandler webClient.DownloadProgressChanged, AddressOf OnBookFileDownloadProgressChanged
            AddHandler webClient.DownloadStringCompleted, AddressOf OnBookFileDownloadStringCompleted

            AddHandler Loaded, AddressOf OnLoaded
        End Sub

        Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
            ' Case 1:
            '      Navigating to this page from "search" pivot or "request" pivot on MainPage,
            '      Or: returning from tombstoning without having yet downloaded the book.

            If appService.State.ContainsKey("downloadBook") Then
                gutenbergBook = TryCast(appService.State("downloadBook"), GutenbergBook)
                catalogInfo.GutenbergBook = gutenbergBook

                ' Test is enough space is available for download
                Dim freeSpace As Long = 0

                Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
                    freeSpace = storage.AvailableFreeSpace
                End Using

                If 1.5 * gutenbergBook.FileSize > freeSpace Then
                    requiredStorageText.Text = (1.5 * gutenbergBook.FileSize / 1024).ToString("N0")
                    availableStorageText.Text = (freeSpace \ 1024).ToString("N0")
                    insufficientSpaceTextBlock.Visibility = Visibility.Visible
                    downloadButton.IsEnabled = False
                Else
                    ' If space is OK, set a warning about a duplicate book
                    duplicateWarningTextBlock.Visibility = If(appSettings.Library.DoesLibraryEntryExist(gutenbergBook.ID), Visibility.Visible, Visibility.Collapsed)
                End If

                ' Case 2:
                '      Navigating back to this page from DisplayTitlePage or ChapterAnalysisPage,
                '      Or: returning from tombstoning after having downloaded the book.

            ElseIf appService.State.ContainsKey("successfulDownload") Then
                gutenbergBook = TryCast(appService.State("successfulDownload"), GutenbergBook)
                catalogInfo.GutenbergBook = gutenbergBook

                bookInfo = BookInfo.Load(gutenbergBook.ID)
                bookTitleTextBlock.Text = bookInfo.DisplayTitle

                downloadButton.IsEnabled = False
                downloadTextBlock.Text = "Download successful!"
                downloadTextBlock.TextAlignment = TextAlignment.Center
                optionsPanel.Visibility = Visibility.Visible

                ' Case 3:
                '      Returning to this page after reading the newly-downloaded book

            Else ' if (appService.State.ContainsKey("successfulDownloadAndRead"))
                Me.NavigationService.GoBack()
            End If

            MyBase.OnNavigatedTo(args)
        End Sub

        Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedFrom(e)
        End Sub

        Private Sub OnLoaded(ByVal sender As Object, ByVal args As RoutedEventArgs)
            ' Scroll to where the buttons are
            scrollViewer.ScrollToBottom()
        End Sub

        Private Sub OnDownloadButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            ' Switch to "cancel" button
            downloadButton.IsEnabled = False
            downloadButton.Visibility = Visibility.Collapsed
            cancelButton.IsEnabled = True
            cancelButton.Visibility = Visibility.Visible

            ' Initialize and start download of RDF file
            downloadTextBlock.Text = Nothing
            progressBar.Value = 0
        _downloadMode = DownloadMode.RdfFile
            Dim rdfFilename As String = String.Format("http://www.gutenberg.org/ebooks/{0}.rdf", gutenbergBook.ID)
            webClient.DownloadStringAsync(New Uri(rdfFilename))
        End Sub

        Private Sub OnCancelButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            ' Disable the button
            cancelButton.IsEnabled = False

            ' Cancel the download
            webClient.CancelAsync()
        End Sub

        Private Sub OnBookFileDownloadProgressChanged(ByVal sender As Object, ByVal args As DownloadProgressChangedEventArgs)
            ' Check if OK to access UI
        If Me.NavigationService.CanGoBack AndAlso _downloadMode = DownloadMode.BookFile Then
            progressBar.Value = args.ProgressPercentage
        End If
        End Sub

        Private Sub OnBookFileDownloadStringCompleted(ByVal sender As Object, ByVal args As DownloadStringCompletedEventArgs)
            If args.Cancelled Then
                DownloadCompleted("Download cancelled.", Nothing, False, True)
            ElseIf args.Error IsNot Nothing Then
                DownloadCompleted(args.Error.Message, Nothing, False, True)
            ElseIf String.IsNullOrEmpty(args.Result) Then
                DownloadCompleted("Downloaded book has no content.", Nothing, False, False)
            Else
            If _downloadMode = DownloadMode.BookFile Then
                DownloadCompleted("Download successful!", args.Result, True, False)
            Else ' downloadMode == DownloadMode.RdfFile
                ' Replace the filename with a UTF-8 file from the RDF
                Dim rdf As RDF = rdf.Deserialize(args.Result)
                Dim filename As String = Nothing

                If GetReplacementFile(rdf, "text/plain; charset=utf-8", filename) Then
                    gutenbergBook.Format = "text/plain; charset=utf-8"
                    gutenbergBook.Filename = filename
                ElseIf GetReplacementFile(rdf, "text/plain", filename) Then
                    gutenbergBook.Format = "text/plain"
                    gutenbergBook.Filename = filename
                End If

                ' Regardless whether a replacement UTF-8 was found or not, download the file
                _downloadMode = DownloadMode.BookFile
                webClient.DownloadStringAsync(New Uri(gutenbergBook.Filename))
            End If
            End If
        End Sub

        Private Function GetReplacementFile(ByVal rdf As RDF, ByVal format As String, <System.Runtime.InteropServices.Out()> ByRef filename As String) As Boolean
        For Each file In rdf.Files
            If file.format.Description.Values.Count = 1 AndAlso file.format.Description.Values(0).content = format Then
                filename = file.about
                Return True
            End If
        Next file
            filename = Nothing
            Return False
        End Function

        Private Sub DownloadCompleted(ByVal message As String, ByVal bookContents As String, ByVal isSuccess As Boolean, ByVal canTryAgain As Boolean)
            ' Check if the page is still active before accessing controls
            If Me.NavigationService.CanGoBack Then
                downloadTextBlock.Text = message

                If isSuccess Then
                    downloadTextBlock.TextAlignment = TextAlignment.Center
                End If

                downloadButton.Visibility = Visibility.Visible
                cancelButton.Visibility = Visibility.Collapsed
                downloadButton.IsEnabled = canTryAgain

                scrollViewer.ScrollToBottom()
            End If

            If isSuccess Then
                ' Save the GutenbergBook object in isolated storage using an extension method.
                gutenbergBook.Save()

                ' Create a new BookInfo (which also saves the bookContents).
                bookInfo = BookInfo.Create(gutenbergBook.ID, bookContents, gutenbergBook.FriendlyTitle)

                ' Save the new BookInfo.
                bookInfo.Save()

                ' Check if the book is already in the library; if so, remove it.
                If appSettings.Library.DoesLibraryEntryExist(gutenbergBook.ID) AndAlso appSettings.Bedside.Contains(bookInfo) Then
                    appSettings.Bedside.Remove(bookInfo)
                End If

                ' Add the new book to the library and save.
                appSettings.Library.CreateLibraryEntry(gutenbergBook.ID, gutenbergBook.Titles, gutenbergBook.Creators)

                ' Set a global application state indicating the file has been successfully downloaded.
                StateDictionaryHelper.SafeRemove(appService.State, "downloadBook")
                StateDictionaryHelper.SafeAdd(appService.State, "successfulDownload", gutenbergBook)

                ' Check if this page is still active before accessing controls
                If Me.NavigationService.CanGoBack Then
                    duplicateWarningTextBlock.Visibility = Visibility.Collapsed
                    optionsPanel.Visibility = Visibility.Visible
                    scrollViewer.ScrollToBottom()
                End If
            End If
        End Sub

        ' These two pages load and save the BookInfo
        Private Sub OnChangeDisplayTitleClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Dim uri As String = String.Format("/DisplayTitlePage.xaml?bookid={0}", bookInfo.ID)
            Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
        End Sub

        Private Sub OnChapterBreaksClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Dim uri As String = String.Format("/ChapterAnalysisPage.xaml?bookid={0}", bookInfo.ID)
            Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
        End Sub

        Private Sub OnDoneButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            Me.NavigationService.GoBack()
        End Sub

        Private Sub OnReadButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
            StateDictionaryHelper.SafeRemove(appService.State, "successfulDownload")
            StateDictionaryHelper.SafeAdd(appService.State, "successfulDownloadAndRead", gutenbergBook.ID)

            TryCast(Application.Current, App).CurrentBook = bookInfo
            Me.NavigationService.Navigate(New Uri("/BookViewerPage.xaml", UriKind.Relative))
        End Sub
    End Class
