﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Petzold.Phone.EBookReader


Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private appService As PhoneApplicationService = PhoneApplicationService.Current
    Private appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
    Private lastPivotSelectedIndex As Integer = -1
    Private libraryScrollOffset As Double = 0

    Public Sub New()
        InitializeComponent()
        besideListBox.ItemsSource = appSettings.Bedside
        libraryItemsControl.ItemsSource = appSettings.Library.Authors
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        StateDictionaryHelper.SafeAdd(Me.State, "lastPivotSelectedIndex", pivot.SelectedIndex)
        StateDictionaryHelper.SafeAdd(Me.State, "libraryScrollOffset", libraryScrollViewer.VerticalOffset)

        For Each pivotItem As PivotItem In pivot.Items
            If TypeOf pivotItem.Content Is ITombstonable Then
                TryCast(pivotItem.Content, ITombstonable).SaveState(pivotItem.Header.ToString(), Me.State)
            End If
        Next pivotItem

        MyBase.OnNavigatedFrom(args)
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        lastPivotSelectedIndex = StateDictionaryHelper.SafeGet(Me.State, "lastPivotSelectedIndex", -1)
        libraryScrollOffset = StateDictionaryHelper.SafeGet(Me.State, "libraryScrollOffset", 0.0)

        Dim book As BookInfo = (TryCast(Application.Current, App)).CurrentBook

        ' If we're navigating back here, and App.CurrentBook is != null,
        '  then we're done reading that book for now, and the BookInfo 
        '  file needs to be saved.
        If book IsNot Nothing Then
            book.Save()
            TryCast(Application.Current, App).CurrentBook = Nothing
        End If

        For Each pivotItem As PivotItem In pivot.Items
            If TypeOf pivotItem.Content Is ITombstonable Then
                TryCast(pivotItem.Content, ITombstonable).RestoreState(pivotItem.Header.ToString(), Me.State)
            End If
        Next pivotItem

        MyBase.OnNavigatedTo(args)
    End Sub

    Private Sub OnPivotLoaded(ByVal sender As Object, ByVal args As RoutedEventArgs)
        ' Program starting up
        If lastPivotSelectedIndex = -1 Then
            ' Go to bedside if books are there
            If appSettings.Bedside.Count <> 0 Then
                pivot.SelectedIndex = 0
                ' Or library if books are there
            ElseIf appSettings.Library.Authors.Count <> 0 Then
                pivot.SelectedIndex = 1
                ' Otherwise go "shopping"
            Else
                pivot.SelectedIndex = 2
            End If
        ElseIf lastPivotSelectedIndex = 0 AndAlso appSettings.Bedside.Count <> 0 Then
            pivot.SelectedIndex = 0
        ElseIf lastPivotSelectedIndex <= 1 AndAlso appSettings.Library.Authors.Count <> 0 Then
            pivot.SelectedIndex = 1
        ElseIf lastPivotSelectedIndex = 4 Then ' about
            pivot.SelectedIndex = 4
        ElseIf lastPivotSelectedIndex = 2 OrElse lastPivotSelectedIndex = 3 Then ' download portals
            ' Coming back from successful download after user has pressed "read book"
            If appService.State.ContainsKey("successfulDownloadAndRead") Then
                appService.State.Remove("successfulDownloadAndRead")
                pivot.SelectedIndex = 0 ' bedside
                ' Coming back from successful download 
            ElseIf appService.State.ContainsKey("successfulDownload") Then
                appService.State.Remove("successfulDownload")
                pivot.SelectedIndex = 1 ' library
                ' Coming back from DownloadBookPage without downloading
            Else
                If appService.State.ContainsKey("downloadFile") Then
                    appService.State.Remove("downloadFile")
                End If

                pivot.SelectedIndex = lastPivotSelectedIndex
            End If
        End If

        ' Wonder why this isn't working?
        libraryScrollViewer.ScrollToVerticalOffset(libraryScrollOffset)
        libraryScrollViewer.UpdateLayout()
    End Sub

    Private Sub OnInfoButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim btn As Button = TryCast(sender, Button)
        Dim bookID As Integer = CInt(Fix(btn.Tag))
        Dim uri As String = String.Format("/BookInformationPage.xaml?bookid={0}", bookID)
        Me.NavigationService.Navigate(New Uri(uri, UriKind.Relative))
    End Sub

    Private Sub OnBedsideListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
        Dim lstbox As ListBox = TryCast(sender, ListBox)
        Dim book As BookInfo = TryCast(lstbox.SelectedItem, BookInfo)
        lstbox.SelectedItem = Nothing

        If book IsNot Nothing Then
            ' Call method to generate paragraphs
            book.Load()
            NavigateToBookViewerPage(book)
        End If
    End Sub

    Private Sub OnLibraryListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
        Dim lstbox As ListBox = TryCast(sender, ListBox)
        Dim title As TitleInfo = TryCast(lstbox.SelectedItem, TitleInfo)
        lstbox.SelectedItem = Nothing

        If title IsNot Nothing Then
            Dim bookID As Integer = title.ID
            NavigateToBookViewerPage(bookID)
        End If
    End Sub

    Private Sub NavigateToBookViewerPage(ByVal bookID As Integer)
        Dim book As BookInfo = BookInfo.Load(bookID)
        NavigateToBookViewerPage(book)
    End Sub

    Private Sub NavigateToBookViewerPage(ByVal book As BookInfo)
        ' Store BookInfo object in App.CurrentBook property for reference by all pages
        TryCast(Application.Current, App).CurrentBook = book

        ' Navigate to the BookViewerPage
        Me.NavigationService.Navigate(New Uri("/BookViewerPage.xaml", UriKind.Relative))
    End Sub
End Class
