﻿Imports System
Imports System.Collections.Generic
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports Microsoft.Phone.Shell
Imports Petzold.Phone.EBookReader
Imports PhreeBookReader.CatalogServiceReference


Partial Public Class RequestControl
    Inherits UserControl
    Implements ITombstonable
    Private proxy As New CatalogServiceClient()
    Private bookID As Integer

    Public Sub New()
        InitializeComponent()
        AddHandler proxy.BookInfoCompleted, AddressOf OnCatalogServiceProxyBookInfoCompleted
    End Sub

    ' ITombstonable method, called from MainPage
    Public Sub SaveState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.SaveState
        StateDictionaryHelper.SafeAdd(state, prefix & "bookIdText", bookIdTextBox.Text)
        StateDictionaryHelper.SafeAdd(state, prefix & "statusText", statusTextBlock.Text)
        StateDictionaryHelper.SafeAdd(state, prefix & "resultsVisibility", resultsPanel.Visibility)
        StateDictionaryHelper.SafeAdd(state, prefix & "gutenbergBook", catalogInfo.GutenbergBook)
    End Sub

    ' ITombstonable method, called from MainPage
    Public Sub RestoreState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.RestoreState
        bookIdTextBox.Text = StateDictionaryHelper.SafeGet(state, prefix & "bookIdText", "")
        statusTextBlock.Text = StateDictionaryHelper.SafeGet(Of String)(state, prefix & "statusText", Nothing)
        resultsPanel.Visibility = StateDictionaryHelper.SafeGet(state, prefix & "resultsVisibility", Visibility.Collapsed)
        catalogInfo.GutenbergBook = StateDictionaryHelper.SafeGet(Of GutenbergBook)(state, prefix & "gutenbergBook", Nothing)
    End Sub

    Private Sub OnBookIdTextBoxTextChanged(ByVal sender As Object, ByVal args As TextChangedEventArgs)
        requestButton.IsEnabled = bookIdTextBox.Text.Length > 0 AndAlso Int32.TryParse(bookIdTextBox.Text.Trim(" "c), bookID) AndAlso bookID > 0
    End Sub

    Protected Overrides Sub OnKeyUp(ByVal args As KeyEventArgs)
        If args.Key = Key.Enter Then
            OnRequestButtonClick(Me, args)
            args.Handled = True
        End If
        MyBase.OnKeyUp(args)
    End Sub

    Private Sub OnRequestButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        StateDictionaryHelper.SafeRemove(PhoneApplicationService.Current.State, "downloadBook")
        EnableForSearch(False)
        statusTextBlock.Text = Nothing
        resultsPanel.Visibility = Visibility.Collapsed
        catalogInfo.GutenbergBook = Nothing
        proxy.BookInfoAsync(bookID)
    End Sub

    Private Sub OnCatalogServiceProxyBookInfoCompleted(ByVal sender As Object, ByVal args As BookInfoCompletedEventArgs)
        EnableForSearch(True)

        If args.Cancelled Then
            statusTextBlock.Text = "Search was cancelled"
        ElseIf args.Error IsNot Nothing Then
            statusTextBlock.Text = args.Error.Message
        ElseIf args.Result Is Nothing Then
            statusTextBlock.Text = "No book found with that ID number"
        Else
            resultsPanel.Visibility = Visibility.Visible
            scrollViewer.ScrollToBottom()
            catalogInfo.GutenbergBook = args.Result
            EnableForSearch(True)
        End If
    End Sub

    Private Sub EnableForSearch(ByVal enable As Boolean)
        requestButton.IsEnabled = enable
        bookIdTextBox.IsEnabled = enable
        progressBar.Visibility = If(enable, Visibility.Collapsed, Visibility.Visible)
        progressBar.IsIndeterminate = Not enable
    End Sub

    Private Sub OnContinueButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim book As GutenbergBook = catalogInfo.GutenbergBook
        StateDictionaryHelper.SafeAdd(PhoneApplicationService.Current.State, "downloadBook", book)
        TryCast(Application.Current.RootVisual, Frame).Navigate(New Uri("/DownloadBookPage.xaml", UriKind.Relative))
    End Sub
End Class
