﻿Imports System
Imports System.Collections.Generic
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Threading
Imports Microsoft.Phone.Shell
Imports Petzold.Phone.EBookReader
Imports PhreeBookReader.CatalogServiceReference


Partial Public Class SearchControl
    Inherits UserControl
    Implements ITombstonable
    Private proxy As New CatalogServiceClient()
    Private catalogStats As CatalogStats = Nothing

    ' Consolidate these three fields for tombstoning
    Public Structure SearchParams
        Public Property Title() As String
        Public Property Author() As String
        Public Property Index() As Integer
        Public Property SearchKey() As String
    End Structure
    Private searchParamsVariable As SearchParams

    Public Sub New()
        InitializeComponent()
        AddHandler proxy.GetStatsCompleted, AddressOf OnCatalogServiceProxyGetStatsCompleted
        AddHandler proxy.SearchCompleted, AddressOf OnCatalogServiceProxySearchCompleted
        proxy.GetStatsAsync()
    End Sub

    ' ITombstonable method, called from MainPage
    Public Sub SaveState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.SaveState
        StateDictionaryHelper.SafeAdd(state, prefix & "titleTextBox", titleTextBox.Text)
        StateDictionaryHelper.SafeAdd(state, prefix & "authorTextBox", authorTextBox.Text)
        StateDictionaryHelper.SafeAdd(state, prefix & "searchParams", searchParamsVariable)
        StateDictionaryHelper.SafeAdd(state, prefix & "statusText", statusTextBlock.Text)
        StateDictionaryHelper.SafeAdd(state, prefix & "nextSearchText", nextSearchButton.Content)
        StateDictionaryHelper.SafeAdd(state, prefix & "nextSearchVisibility", nextSearchButton.Visibility)

        If resultsListBox.Items.Count <> 0 Then
            Dim items As New List(Of GutenbergBook)()

            For Each item In resultsListBox.Items
                items.Add(TryCast(item, GutenbergBook))
            Next item

            Dim tombstoneObjects As IDictionary(Of String, Object) = (TryCast(Application.Current, App)).TombstoneObjects
            StateDictionaryHelper.SafeAdd(tombstoneObjects, prefix & "listBoxItems", items)
        End If
    End Sub

    ' ITombstonable method, called from MainPage
    Public Sub RestoreState(ByVal prefix As String, ByVal state As IDictionary(Of String, Object)) Implements ITombstonable.RestoreState
        titleTextBox.Text = StateDictionaryHelper.SafeGet(state, prefix & "titleTextBox", "")
        authorTextBox.Text = StateDictionaryHelper.SafeGet(state, prefix & "authorTextBox", "")
        searchParamsVariable = StateDictionaryHelper.SafeGet(state, prefix & "searchParams", New SearchParams())
        statusTextBlock.Text = StateDictionaryHelper.SafeGet(Of String)(state, prefix & "statusText", Nothing)
        nextSearchButton.Content = StateDictionaryHelper.SafeGet(state, prefix & "nextSearchText", "search")
        nextSearchButton.Visibility = StateDictionaryHelper.SafeGet(state, prefix & "nextSearchVisibility", Visibility.Collapsed)

        ' Only restore ListBox contents if we need to
        If resultsListBox.Items.Count = 0 Then
            Dim tombstoneObjects As IDictionary(Of String, Object) = (TryCast(Application.Current, App)).TombstoneObjects
            Dim items As List(Of GutenbergBook) = StateDictionaryHelper.SafeGet(tombstoneObjects, prefix & "listBoxItems", New List(Of GutenbergBook)())

            For Each item In items
                resultsListBox.Items.Add(item)
            Next item
        End If
    End Sub

    Private Sub OnCatalogServiceProxyGetStatsCompleted(ByVal sender As Object, ByVal args As GetStatsCompletedEventArgs)
        If args.Cancelled Then
            Return
        ElseIf args.Error IsNot Nothing Then
            ' Try again in 10 seconds
            Dim tmr As DispatcherTimer = New DispatcherTimer With {.Interval = TimeSpan.FromSeconds(10)}
            AddHandler tmr.Tick, Sub(tmrSender, tmrArgs)
                                     tmr.Stop()
                                     proxy.GetStatsAsync()
                                 End Sub
            tmr.Start()
        ElseIf catalogInfoTextBlock IsNot Nothing Then
            catalogStats = args.Result
            catalogInfoTextBlock.Text = String.Format("Catalog dated {0} lists {1} books", catalogStats.CatalogDate, catalogStats.BookCount)
        End If
    End Sub

    Private Sub OnTextBoxTextChanged(ByVal sender As Object, ByVal args As TextChangedEventArgs)
        searchButton.IsEnabled = titleTextBox.Text.Length > 0 OrElse authorTextBox.Text.Length > 0
    End Sub

    Protected Overrides Sub OnKeyUp(ByVal args As KeyEventArgs)
        If args.Key = Key.Enter Then
            OnSearchButtonClick(Me, args)
            args.Handled = True
        End If
        MyBase.OnKeyUp(args)
    End Sub

    Private Sub OnSearchButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        EnableForSearch(False)
        statusTextBlock.Text = Nothing
        nextSearchButton.Visibility = Visibility.Collapsed
        resultsListBox.Items.Clear()

        searchParamsVariable.Title = titleTextBox.Text
        searchParamsVariable.Author = authorTextBox.Text
        searchParamsVariable.Index = 0

        proxy.GetStatsAsync()
        proxy.SearchAsync(searchParamsVariable.Title, searchParamsVariable.Author, searchParamsVariable.Index, Nothing)
    End Sub

    Private Sub OnCatalogServiceProxySearchCompleted(ByVal sender As Object, ByVal args As SearchCompletedEventArgs)
        EnableForSearch(True)

        If args.Cancelled Then
            statusTextBlock.Text = "Search was cancelled"
        ElseIf args.Error IsNot Nothing Then
            statusTextBlock.Text = args.Error.Message
        Else
            Dim searchResults As SearchResults = args.Result

            ' Check if catalog stats needs to be updated
            If catalogStats IsNot Nothing AndAlso catalogStats.SearchKey <> searchResults.SearchKey Then
                proxy.GetStatsAsync()
            End If

            If searchResults.Books Is Nothing OrElse searchResults.Books.Count = 0 Then
                statusTextBlock.Text = "No matches found"
                nextSearchButton.Visibility = Visibility.Collapsed
            Else
                For Each book In searchResults.Books
                    resultsListBox.Items.Add(book)
                Next book

                Dim returnCount As Integer = searchResults.Books.Count
                Dim listCount As Integer = searchResults.StartIndex + returnCount
                Dim totalCount As Integer = searchResults.TotalCount

                If listCount < totalCount Then
                    statusTextBlock.Text = String.Format("Showing first {0} matches out of {1}", listCount, totalCount)

                    nextSearchButton.Visibility = Visibility.Visible
                    nextSearchButton.IsEnabled = True
                    searchParamsVariable.Index = listCount
                    searchParamsVariable.SearchKey = searchResults.SearchKey

                    If returnCount + listCount < totalCount Then
                        nextSearchButton.Content = String.Format("get next {0} {1}", returnCount, MatchesText(returnCount))
                    Else
                        Dim lastCount As Integer = totalCount - listCount
                        nextSearchButton.Content = String.Format("get last {0} {1}", lastCount, MatchesText(lastCount))
                    End If
                Else
                    statusTextBlock.Text = String.Format("Showing {0} {1}", listCount, MatchesText(listCount))
                    nextSearchButton.Visibility = Visibility.Collapsed
                End If
            End If

            statusTextBlock.Text += String.Format(" ({0} seconds)", (searchResults.FinishTime - searchResults.StartTime).TotalSeconds.ToString("F1"))
        End If
    End Sub

    Private Function MatchesText(ByVal num As Integer) As String
        Return If(num = 1, "match", "matches")
    End Function

    Private Sub OnNextSearchButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        EnableForSearch(False)
        nextSearchButton.IsEnabled = False

        proxy.SearchAsync(searchParamsVariable.Title, searchParamsVariable.Author, searchParamsVariable.Index, searchParamsVariable.SearchKey)
    End Sub

    Private Sub EnableForSearch(ByVal enable As Boolean)
        searchButton.IsEnabled = enable
        titleTextBox.IsEnabled = enable
        authorTextBox.IsEnabled = enable
        progressBar.Visibility = If(enable, Visibility.Collapsed, Visibility.Visible)
        progressBar.IsIndeterminate = Not enable
    End Sub

    Private Sub OnResultsListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
        If resultsListBox.SelectedItem IsNot Nothing Then
            Dim book As GutenbergBook = TryCast(resultsListBox.SelectedItem, GutenbergBook)
            resultsListBox.SelectedItem = Nothing

            StateDictionaryHelper.SafeAdd(PhoneApplicationService.Current.State, "downloadBook", book)
            TryCast(Application.Current.RootVisual, Frame).Navigate(New Uri("/DownloadBookPage.xaml", UriKind.Relative))
        End If
    End Sub
End Class
