﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Petzold.Phone.EBookReader


Partial Public Class SettingsPage
    Inherits PhoneApplicationPage
    Private appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
    Private initializingTransitionButtons As Boolean

    Public Sub New()
        InitializeComponent()
        AddHandler Loaded, Sub(sender, args)
                               fontFamilyListBox.SelectedItem = appSettings.FontFamily
                               fontSizeSlider.Value = CInt(Fix(10 * appSettings.FontSizeMultiplier))
                               fontFamilyListBox.ScrollIntoView(fontFamilyListBox.SelectedItem)
                           End Sub
    End Sub

    Private Sub OnLoadedPivotItem(ByVal sender As Object, ByVal e As PivotItemEventArgs)
        If TryCast(e.Item.Header, String) = "transitions" Then
            initializingTransitionButtons = True

            For Each child In transitionRadioButtonStack.Children
                Dim radio As RadioButton = TryCast(child, RadioButton)
                radio.IsChecked = TryCast(radio.Content, String) = appSettings.PageTransition
            Next child
            bookViewer.JumpToPage(0, 0)
            initializingTransitionButtons = False
        End If
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal arg As NavigationEventArgs)
        appSettings.FontFamily = TryCast(fontFamilyListBox.SelectedItem, String)
        appSettings.FontSizeMultiplier = fontSizeSlider.Value / 10.0
        MyBase.OnNavigatedFrom(arg)
    End Sub

    Private Sub OnFontFamilyListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
        PossiblyDisableDefaultsButton()
    End Sub

    Private Sub OnFontSizeSliderValueChanged(ByVal sender As Object, ByVal args As RoutedPropertyChangedEventArgs(Of Integer))
        PossiblyDisableDefaultsButton()
    End Sub

    Private Sub PossiblyDisableDefaultsButton()
        If restoreDefaultsButton IsNot Nothing AndAlso fontFamilyListBox IsNot Nothing AndAlso fontSizeSlider IsNot Nothing Then
            restoreDefaultsButton.IsEnabled = (TryCast(fontFamilyListBox.SelectedItem, String) <> AppSettings.DefaultFontFamily OrElse fontSizeSlider.Value / 10.0 <> AppSettings.DefaultFontSizeMultiplier)
        End If
    End Sub

    Private Sub OnRestoreDefaultsButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        fontFamilyListBox.SelectedItem = AppSettings.DefaultFontFamily
        fontSizeSlider.Value = CInt(Fix(10 * AppSettings.DefaultFontSizeMultiplier))
    End Sub

    Private Sub OnTransitionRadioButtonChecked(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim radio As RadioButton = TryCast(sender, RadioButton)
        appSettings.PageTransition = TryCast(radio.Content, String)

        Select Case appSettings.PageTransition
            Case "slide"
                bookViewer.PageTransition = New SlideTransition()
            Case "skew"
                bookViewer.PageTransition = New SkewTransition()
            Case "curl"
                bookViewer.PageTransition = New CurlTransition()
            Case "flip"
                bookViewer.PageTransition = New FlipTransition()
        End Select
        If Not initializingTransitionButtons Then
            bookViewer.AnimateToNextPage()
        End If
    End Sub
End Class
